@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/auxtext.c,v 1.9 2009/09/30 14:53:55 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/auxtext.c,v 1.15 2009/11/09 17:05:05 cvsps Exp $
  */
 
 #define _INCLUDE_FROM_AUXTEXT_C_
@@ -1379,6 +1379,57 @@ void DumpTextPath(FP, ObjPtr)
    fprintf(FP, "   %s\n", gPsCmd[PS_GRESTORE]);
 }
 
+static
+void DumpSimpleTextBBoxPath(FP, just, x, lty, rby, indent)
+   FILE *FP;
+   int just, x, lty, rby, indent;
+{
+   int i=0;
+
+   if (psUseShortHand) {
+      for (i=0; i < indent; i++) fprintf(FP, " ");
+      fprintf(FP, "%s %1d %1d %s ",
+            gPsCmd[PS_NEWPATH],
+            x, lty, gPsCmd[PS_MOVETO]);
+      switch (just) {
+      case JUST_L: break;
+      case JUST_C:
+         fprintf(FP, "tgifstrw 2 %s %s 0 %s ",
+               gPsCmd[PS_DIV], gPsCmd[PS_NEG], gPsCmd[PS_RMOVETO]);
+         break;
+      case JUST_R:
+         fprintf(FP, "tgifstrw %s 0 %s ", gPsCmd[PS_NEG], gPsCmd[PS_RMOVETO]);
+         break;
+      }
+      fprintf(FP, "tgifstrw 0 %s 0 %1d %s tgifstrw neg 0 %s %s ",
+            gPsCmd[PS_RLINETO], rby-lty, gPsCmd[PS_RLINETO],
+            gPsCmd[PS_RLINETO], gPsCmd[PS_CLOSEPATH]);
+   } else {
+      for (i=0; i < indent; i++) fprintf(FP, " ");
+      fprintf(FP, "%s\n", gPsCmd[PS_NEWPATH]);
+      for (i=0; i < indent; i++) fprintf(FP, " ");
+
+      fprintf(FP, "   %1d %1d %s ", x, lty, gPsCmd[PS_MOVETO]);
+      switch (just) {
+      case JUST_L: break;
+      case JUST_C:
+         fprintf(FP, "tgifstrw 2 %s %s 0 %s\n",
+               gPsCmd[PS_DIV], gPsCmd[PS_NEG], gPsCmd[PS_RMOVETO]);
+         for (i=0; i < indent+3; i++) fprintf(FP, " ");
+         break;
+      case JUST_R:
+         fprintf(FP, "tgifstrw %s 0 %s\n", gPsCmd[PS_NEG], gPsCmd[PS_RMOVETO]);
+         for (i=0; i < indent+3; i++) fprintf(FP, " ");
+         break;
+      }
+      fprintf(FP, "tgifstrw 0 %s ", gPsCmd[PS_RLINETO]);
+      fprintf(FP, "0 %1d %s ", rby-lty, gPsCmd[PS_RLINETO]);
+      fprintf(FP, "tgifstrw neg 0 %s\n", gPsCmd[PS_RLINETO]);
+      for (i=0; i < indent; i++) fprintf(FP, " ");
+      fprintf(FP, "%s ", gPsCmd[PS_CLOSEPATH]);
+   }
+}
+
 static
 void DumpTextFill(FP, ObjPtr)
    FILE *FP;
@@ -1387,7 +1438,12 @@ void DumpTextFill(FP, ObjPtr)
    struct TextRec *text_ptr=ObjPtr->detail.t;
    int fill=text_ptr->fill, trans_pat=ObjPtr->trans_pat, ltx, lty, rbx, rby;
    int color_index=ObjPtr->color;
+   int simple_text=SimpleTextObj(text_ptr);
 
+   if (simple_text) {
+      fprintf(FP, "   /tgifstrw 0 def\n");
+      DumpSimpleTextPath(FP, ObjPtr);
+   }
    DumpRGBColorLine(FP, color_index, 0, TRUE);
 
    if (ObjPtr->ctm == NULL) {
@@ -1421,12 +1477,22 @@ void DumpTextFill(FP, ObjPtr)
    switch (fill) {
    case NONEPAT: break;
    case SOLIDPAT:
-      DumpRectPath(FP, ltx, lty, rbx, rby, 0, FALSE);
+      if (simple_text) {
+         DumpSimpleTextBBoxPath(FP, text_ptr->minilines.just,
+               ObjPtr->x, lty, rby, 0);
+      } else {
+         DumpRectPath(FP, ltx, lty, rbx, rby, 0, FALSE);
+      }
       fprintf(FP, "%s\n", gPsCmd[PS_FILL]);
       break;
    case BACKPAT:
       if (!trans_pat) {
-         DumpRectPath(FP, ltx, lty, rbx, rby, 0, FALSE);
+         if (simple_text) {
+            DumpSimpleTextBBoxPath(FP, text_ptr->minilines.just,
+                  ObjPtr->x, lty, rby, 0);
+         } else {
+            DumpRectPath(FP, ltx, lty, rbx, rby, 0, FALSE);
+         }
          fprintf(FP, "1 %s %s\n", gPsCmd[PS_SETGRAY], gPsCmd[PS_FILL]);
          DumpRGBColorLine(FP, color_index, 0, TRUE);
       }
@@ -1436,7 +1502,12 @@ void DumpTextFill(FP, ObjPtr)
       fprintf(FP, "%s\n", gPsCmd[PS_GSAVE]);
       if (colorDump || !useGray) {
          if (!trans_pat) {
-            DumpRectPath(FP, ltx, lty, rbx, rby, 3, FALSE);
+            if (simple_text) {
+               DumpSimpleTextBBoxPath(FP, text_ptr->minilines.just,
+                     ObjPtr->x, lty, rby, 3);
+            } else {
+               DumpRectPath(FP, ltx, lty, rbx, rby, 3, FALSE);
+            }
             fprintf(FP, "1 %s %s\n", gPsCmd[PS_SETGRAY], gPsCmd[PS_FILL]);
             DumpRGBColorLine(FP, color_index, 3, TRUE);
          }
@@ -1446,11 +1517,21 @@ void DumpTextFill(FP, ObjPtr)
       }
       if (colorDump || !useGray) {
          if (preDumpSetup) PSUseColorPattern();
-         DumpRectPath(FP, ltx, lty, rbx, rby, 3, FALSE);
+         if (simple_text) {
+            DumpSimpleTextBBoxPath(FP, text_ptr->minilines.just,
+                  ObjPtr->x, lty, rby, 3);
+         } else {
+            DumpRectPath(FP, ltx, lty, rbx, rby, 3, FALSE);
+         }
          fprintf(FP, "%s %s\n", gPsCmd[PS_EOCLIP], gPsCmd[PS_NEWPATH]);
          DumpPatFill(FP, fill, ObjPtr->bbox, 3, TRUE);
       } else {
-         DumpRectPath(FP, ltx, lty, rbx, rby, 3, FALSE);
+         if (simple_text) {
+            DumpSimpleTextBBoxPath(FP, text_ptr->minilines.just,
+                  ObjPtr->x, lty, rby, 3);
+         } else {
+            DumpRectPath(FP, ltx, lty, rbx, rby, 3, FALSE);
+         }
          fprintf(FP, "%s\n", gPsCmd[PS_FILL]);
       }
       fprintf(FP, "%s\n", gPsCmd[PS_GRESTORE]);
@@ -1467,6 +1548,7 @@ void DumpTextObj(FP, ObjPtr)
    MiniLinesInfo *minilines=(&text_ptr->minilines);
    int pen=text_ptr->pen, fill=text_ptr->fill, trans_pat=ObjPtr->trans_pat;
    int underline_y_offset=text_ptr->underline_y_offset;
+   int overline_y_offset=text_ptr->overline_y_offset;
 
    if ((fill == NONEPAT || (trans_pat && fill == BACKPAT)) &&
          (pen == NONEPAT || (trans_pat && pen == BACKPAT))) {
@@ -1488,6 +1570,7 @@ void DumpTextObj(FP, ObjPtr)
    transPat = trans_pat;
    objFill = fill;
    curUnderlineYOffset = underline_y_offset;
+   curOverlineYOffset = overline_y_offset;
 
    if (PRTGIF || text_ptr->read_only) {
       canvasFontAsc = minilines->first->asc;
@@ -1654,6 +1737,7 @@ void SaveStrSeg(FP, pStrSeg)
     */
 {
    char font_str[MAXSTRING];
+   int overunder=((pStrSeg->overline_on<<1)+pStrSeg->underline_on);
 
    GetPSFontStr(pStrSeg->font, pStrSeg->style, font_str);
          /* font_str starts with the '/' character */
@@ -1669,7 +1753,7 @@ void SaveStrSeg(FP, pStrSeg)
          "'%s',%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,",
          &font_str[1], pStrSeg->style, pStrSeg->sz_unit,
          pStrSeg->w, pStrSeg->asc, pStrSeg->des, pStrSeg->min_lbearing,
-         pStrSeg->max_rextra, (pStrSeg->underline_on != 0),
+         pStrSeg->max_rextra, overunder,
          pStrSeg->double_byte, pStrSeg->double_byte_mod_bytes,
          pStrSeg->double_byte_vertical, pStrSeg->direction) == EOF) {
       writeFileFailed = TRUE;
@@ -1840,13 +1924,43 @@ void SaveMiniLines(FP, minilines)
    if (fprintf(FP, "\n])") == EOF) writeFileFailed = TRUE;
 }
 
+void CombineOverUnderYOffsets(underline_y_offset, overline_y_offset,
+      pn_combined)
+   int underline_y_offset, overline_y_offset, *pn_combined;
+{
+   underline_y_offset = (underline_y_offset & 0x0ffff);
+   overline_y_offset = (overline_y_offset & 0x0ffff);
+   *pn_combined = ((overline_y_offset << 16) | underline_y_offset);
+}
+
+void UncombineOverUnderYOffsets(combined, pn_underline_y_offset,
+      pn_overline_y_offset)
+   int combined, *pn_underline_y_offset, *pn_overline_y_offset;
+{
+   int underline_y_offset=(combined & 0x0ffff);
+   int overline_y_offset=((combined >> 16) & 0x0ffff);
+
+   if ((underline_y_offset & 0x08000) == 0x08000) {
+      underline_y_offset = (0x010000 - abs(underline_y_offset));
+      underline_y_offset = -underline_y_offset;
+   }
+   if ((overline_y_offset & 0x08000) == 0x08000) {
+      overline_y_offset = (0x010000 - abs(overline_y_offset));
+      overline_y_offset = -overline_y_offset;
+   }
+   *pn_underline_y_offset = underline_y_offset;
+   *pn_overline_y_offset = overline_y_offset;
+}
+
 void SaveTextObj(FP, ObjPtr)
    FILE *FP;
    struct ObjRec *ObjPtr;
 {
    struct TextRec *text_ptr=ObjPtr->detail.t;
-   int compressed=FALSE;
+   int compressed=FALSE, combined_y_offset=0;
 
+   CombineOverUnderYOffsets(text_ptr->underline_y_offset,
+         text_ptr->overline_y_offset, &combined_y_offset);
    if (fprintf(FP, "text('%s',", colorMenuItems[ObjPtr->color]) == EOF) {
       writeFileFailed = TRUE;
    }
@@ -1862,7 +1976,7 @@ void SaveTextObj(FP, ObjPtr)
    if (fprintf(FP,
          "%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,\"\",",
          text_ptr->fill, text_ptr->minilines.v_space, ObjPtr->rotation,
-         ObjPtr->locked, text_ptr->underline_y_offset, text_ptr->w,
+         ObjPtr->locked, combined_y_offset, text_ptr->w,
          text_ptr->h, text_ptr->min_lbearing, text_ptr->max_rextra) == EOF) {
       writeFileFailed = TRUE;
    }
