@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 2001-2003, William Chia-Wei Cheng.
+ * Copyright (C) 2001-2009, William Chia-Wei Cheng.
  *
  * This file may be distributed under the terms of the Q Public License
  * as defined by Trolltech AS of Norway and appearing in the file
@@ -15,7 +15,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/box.c,v 1.3 2005/01/15 00:23:22 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/box.c,v 1.10 2009/09/28 17:43:48 william Exp $
  */
 
 #define _INCLUDE_FROM_BOX_C_
@@ -114,7 +114,7 @@ void DumpBoxPath(FP, ObjPtr, LtX, LtY, RbX, RbY, Width, Pen, Dash, TransPat)
       if (colorDump || !useGray) {
          if (preDumpSetup) PSUseColorPattern();
          fprintf(FP, "      %s\n", gPsCmd[PS_FLATTENPATH]);
-         DumpPatFill(FP, Pen, 8, ObjPtr->bbox, 6, TRUE);
+         DumpPatFill(FP, Pen, ObjPtr->bbox, 6, TRUE);
       } else {
          fprintf(FP, "      %s\n", gPsCmd[PS_STROKE]);
       }
@@ -198,7 +198,7 @@ void DumpBoxObj(FP, ObjPtr)
             }
             DumpRectPath(FP, ltx, lty, rbx, rby, 3, FALSE);
             fprintf(FP, "%s %s\n", gPsCmd[PS_EOCLIP], gPsCmd[PS_NEWPATH]);
-            DumpPatFill(FP, fill, 8, ObjPtr->bbox, 3, TRUE);
+            DumpPatFill(FP, fill, ObjPtr->bbox, 3, TRUE);
          } else {
             GrayCheck(fill);
             fprintf(FP, "   %s %s\n", GrayStr(fill), gPsCmd[PS_SETGRAY]);
@@ -306,7 +306,7 @@ int CreateBoxObj(X1, Y1, X2, Y2, CreateAbsolute)
 {
    struct BoxRec *box_ptr;
    struct ObjRec *obj_ptr;
-   int width, w, ltx, lty, rbx, rby;
+   int width=0, w, ltx, lty, rbx, rby;
 
    if (X1 == X2 && Y1 == Y2) return FALSE;
    box_ptr = (struct BoxRec *)malloc(sizeof(struct BoxRec));
@@ -355,6 +355,10 @@ int CreateBoxObj(X1, Y1, X2, Y2, CreateAbsolute)
    obj_ptr->bbox.rby += w;
    obj_ptr->type = OBJ_BOX;
    obj_ptr->color = colorIndex;
+   if (mainDisplay != NULL) {
+      UtilStrCpyN(obj_ptr->color_str, sizeof(obj_ptr->color_str),
+            colorMenuItems[colorIndex]);
+   }
    obj_ptr->id = objId++;
    obj_ptr->dirty = FALSE;
    obj_ptr->rotation = 0;
@@ -716,6 +720,7 @@ void ReadBoxObj(FP, Inbuf, ObjPtr)
    (*ObjPtr)->x = ltx;
    (*ObjPtr)->y = lty;
    (*ObjPtr)->color = QuickFindColorIndex(*ObjPtr, color_str, &new_alloc, TRUE);
+   UtilStrCpyN((*ObjPtr)->color_str, sizeof((*ObjPtr)->color_str), color_str);
    (*ObjPtr)->dirty = FALSE;
    (*ObjPtr)->id = id;
    (*ObjPtr)->rotation = rotation;
@@ -740,7 +745,7 @@ void ReadBoxObj(FP, Inbuf, ObjPtr)
       char inbuf[MAXSTRING+1];
       struct XfrmMtrxRec *ctm;
 
-      fgets(inbuf, MAXSTRING, FP);
+      (void)fgets(inbuf, MAXSTRING, FP);
       scanLineNum++;
       InitScan(inbuf, "\t\n, ");
 
