@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/choice.c,v 1.12 2009/08/04 04:39:24 cvsps Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/choice.c,v 1.14 2009/10/22 03:38:31 cvsps Exp $
  */
 
 #define _INCLUDE_FROM_CHOICE_C_
@@ -487,6 +487,11 @@ void ShowCurFont()
       XDrawLine(mainDisplay, choiceWindow, choiceGC, x, y+curUnderlineYOffset,
             x+w, y+curUnderlineYOffset);
    }
+   if (curOverlineOn) {
+      XDrawLine(mainDisplay, choiceWindow, choiceGC,
+            x, y-canvasFontAsc-curOverlineYOffset,
+            x+w, y-canvasFontAsc-curOverlineYOffset);
+   }
    values.foreground = xorOne;
    values.background = xorZero;
    XChangeGC(mainDisplay, choiceGC, GCForeground | GCBackground, &values);
