@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/const.h,v 1.20 2009/07/28 14:33:55 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/const.h,v 1.23 2009/10/21 23:56:33 cvsps Exp $
  */
 
 #ifndef _TGIF_CONST_H_
@@ -382,6 +382,8 @@
 #define PUSH_UNDERLINE_ON 10
 #define PUSH_UNDERLINE_Y_OFFSET 11
 #define PUSH_DOUBLEBYTE 12
+#define PUSH_OVERLINE_ON 13
+#define PUSH_OVERLINE_Y_OFFSET 14
 
 #define NUM_SZ_UNIT_PER_FONT_SIZE 5760
 #define NUM_SZ_UNIT_PER_POINT_SIZE 10240
@@ -479,6 +481,9 @@
 
 /* measurement */
 
+#define TIK_PER_PIXEL_SHIFTS 12
+#define TIK_PER_PIXEL (1<<TIK_PER_PIXEL_SHIFTS)
+
 #define ENGLISH_GRID 0
 #define METRIC_GRID 1
 
@@ -996,6 +1001,7 @@
 #define PROP_MASK_TEXT_STYLE	0x00008000 /* ChangeEditTextProperty() only */
 #define PROP_MASK_VSPACE	0x00010000
 #define PROP_MASK_UNDERLINE_ON	0x00020000
+#define PROP_MASK_OVERLINE_ON	0x00040000
 #define PROP_MASK_WIDTH_INDEX	0x00100000 /* GetProperty() only */
 
 /* mark stuff */
