@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 2001-2006, William Chia-Wei Cheng.
+ * Copyright (C) 2001-2009, William Chia-Wei Cheng.
  *
  * This file may be distributed under the terms of the Q Public License
  * as defined by Trolltech AS of Norway and appearing in the file
@@ -15,7 +15,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/const.h,v 1.4 2006/02/10 23:21:19 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/const.h,v 1.23 2009/10/21 23:56:33 cvsps Exp $
  */
 
 #ifndef _TGIF_CONST_H_
@@ -172,12 +172,17 @@
 
 #define MARK(W,GC,X,Y) XFillRectangle(mainDisplay,W,GC, \
       (X)-handleSize,(Y)-handleSize,(handleSize<<1)+1,(handleSize<<1)+1)
+#define MARKHR(W,GC,X,Y) XDrawRectangle(mainDisplay,W,GC, \
+      (X)-handleSize,(Y)-handleSize,(handleSize<<1),(handleSize<<1))
 #define MARKV(W,GC,X,Y) XDrawLine(mainDisplay,W,GC, \
       (X)-(handleSize+1),(Y),(X)+(handleSize+1),(Y)); \
       XDrawLine(mainDisplay,W,GC,(X),(Y)-(handleSize+1),(X),(Y)+(handleSize+1))
 #define MARKO(W,GC,X,Y) XFillArc(mainDisplay,W,GC, \
       (X)-handleSize,(Y)-handleSize,(handleSize<<1)+1,(handleSize<<1)+1, \
       0,(360<<6))
+#define MARKHO(W,GC,X,Y) XDrawArc(mainDisplay,W,GC, \
+      (X)-handleSize,(Y)-handleSize,(handleSize<<1),(handleSize<<1), \
+      0,(360<<6))
 #define PtInMark(PtX,PtY,MarkX,MarkY) ((PtX) >= (MarkX)-handleSize && \
       (PtY) >= (MarkY)-handleSize && (PtX) <= (MarkX)+handleSize && \
       (PtY) <= (MarkY)+handleSize)
@@ -273,8 +278,9 @@
 #define LT_STRAIGHT 0
 #define LT_SPLINE 1
 #define LT_INTSPLINE 2
+#define LT_STRUCT_SPLINE 3
 
-#define MAXLINETYPES 3
+#define MAXLINETYPES 4
 
 #define LS_PLAIN 0
 #define LS_RIGHT 1
@@ -363,6 +369,8 @@
 #define PUSH_UNDERLINE_ON 10
 #define PUSH_UNDERLINE_Y_OFFSET 11
 #define PUSH_DOUBLEBYTE 12
+#define PUSH_OVERLINE_ON 13
+#define PUSH_OVERLINE_Y_OFFSET 14
 
 #define NUM_SZ_UNIT_PER_FONT_SIZE 5760
 #define NUM_SZ_UNIT_PER_POINT_SIZE 10240
@@ -454,11 +462,15 @@
 #define JPEG_FILE 11
 #define PPM_FILE 12
 #define NETLIST_FILE 13
+#define SVG_FILE 14
 
-#define MAXDEFWHERETOPRINT 14
+#define MAXDEFWHERETOPRINT 15
 
 /* measurement */
 
+#define TIK_PER_PIXEL_SHIFTS 12
+#define TIK_PER_PIXEL (1<<TIK_PER_PIXEL_SHIFTS)
+
 #define ENGLISH_GRID 0
 #define METRIC_GRID 1
 
@@ -696,8 +708,13 @@
 
 #define XPM_XPM  0
 #define XPM_JPEG 1
-#define XPM_EPS  2 /* not implemented */
-#define XPM_EPSI 3 /* not implemented */
+#define PPM_TRUE 2
+#define XPM_EPS  3 /* not implemented */
+#define XPM_EPSI 4 /* not implemented */
+
+#define PPM_DATA_RAW      0 /* not used */
+#define PPM_JPEG_COMPRESS 1
+#define PPM_DATA_DEFLATED 2
 
 /* cmd stuff */
 
@@ -741,6 +758,7 @@
 /* copypaste stuff */
 
 #define TGIF_HEADER 0x80
+#define TGIF_PROTOCOL_ATOM "_TGIF_PROTOCOL_ATOM"
 
 /* interrupt stuff */
 
@@ -970,6 +988,19 @@
 #define PROP_MASK_TEXT_STYLE	0x00008000 /* ChangeEditTextProperty() only */
 #define PROP_MASK_VSPACE	0x00010000
 #define PROP_MASK_UNDERLINE_ON	0x00020000
+#define PROP_MASK_OVERLINE_ON	0x00040000
 #define PROP_MASK_WIDTH_INDEX	0x00100000 /* GetProperty() only */
 
+/* mark stuff */
+
+#define FORWARD 0
+#define REVERSE 1
+
+/* auto rotate pivot stuff */
+
+#define AUTO_ROTATE_PIVOT 0
+#define USER_ROTATE_PIVOT 1
+
+#define MAX_ROTATE_PIVOT 2
+
 #endif /*_TGIF_CONST_H_*/
