@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2006, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2009, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/dialog.c,v 1.3 2006/02/10 23:21:19 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/dialog.c,v 1.8 2009/08/13 01:14:20 cvsps Exp $
  */
 
 #define _INCLUDE_FROM_DIALOG_C_
@@ -91,6 +91,10 @@ unsigned int DrawWindowLoop(OrigX, OrigY, cursor, snap_to_grid)
    int *OrigX, *OrigY, snap_to_grid;
    Cursor cursor;
 {
+   Window focus_win=None;
+   int revert_to=0;
+
+   XGetInputFocus(mainDisplay, &focus_win, &revert_to);
    XGrabPointer(mainDisplay, drawWindow, False,
          PointerMotionMask | ButtonPressMask,
          GrabModeAsync, GrabModeAsync, None, cursor, CurrentTime);
@@ -104,6 +108,10 @@ unsigned int DrawWindowLoop(OrigX, OrigY, cursor, snap_to_grid)
          ExposeEventHandler(&input, TRUE);
       } else if (input.type == ButtonPress) {
          XUngrabPointer(mainDisplay, CurrentTime);
+         if (focus_win != mainWindow) {
+            XSetInputFocus(mainDisplay, mainWindow, RevertToPointerRoot,
+                  CurrentTime);
+         }
          XSync(mainDisplay, False);
          *OrigX = input.xbutton.x;
          *OrigY = input.xbutton.y;
@@ -111,6 +119,10 @@ unsigned int DrawWindowLoop(OrigX, OrigY, cursor, snap_to_grid)
       } else if (input.type == KeyPress) {
          if (KeyPressEventIsEscape(&input.xkey)) {
             XUngrabPointer(mainDisplay, CurrentTime);
+            if (focus_win != mainWindow) {
+               XSetInputFocus(mainDisplay, mainWindow, RevertToPointerRoot,
+                     CurrentTime);
+            }
             XSync(mainDisplay, False);
             return (unsigned int)(-1);
          }
@@ -133,9 +145,14 @@ unsigned int PickAPoint(OrigX, OrigY, cursor)
 {
    XEvent input;
 
-   XGrabPointer(mainDisplay, drawWindow, False, ButtonPressMask,
-         GrabModeAsync, GrabModeAsync, None, cursor, CurrentTime);
-
+#ifdef _TGIF_DBG
+   if (!debugNoPointerGrab) {
+#endif /* _TGIF_DBG */
+      XGrabPointer(mainDisplay, drawWindow, False, ButtonPressMask,
+            GrabModeAsync, GrabModeAsync, None, cursor, CurrentTime);
+#ifdef _TGIF_DBG
+   }
+#endif /* _TGIF_DBG */
    for (;;) {
       XNextEvent(mainDisplay, &input);
 
@@ -430,7 +447,7 @@ void CalcSimpleGeometry(MBInfoPtr, Message)
    MBInfoPtr->msg_win_h = max_h;
    MBInfoPtr->icon_win_w = ICON_W;
    MBInfoPtr->icon_win_h = ICON_H;
-   if (info_bits) { }
+   if (info_bits == NULL) { }
    if (MBInfoPtr->msg_win_h > MBInfoPtr->icon_win_h) {
       MBInfoPtr->icon_win_h = MBInfoPtr->msg_win_h;
    } else {
