@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2006, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2009, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/dup.c,v 1.5 2006/02/10 23:21:19 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/dup.c,v 1.12 2009/06/20 04:14:41 cvsps Exp $
  */
 
 #define _INCLUDE_FROM_DUP_C_
@@ -162,10 +162,10 @@ void DupObjBasics(FromObjPtr, ToObjPtr)
    ToObjPtr->y = FromObjPtr->y;
    ToObjPtr->color = FromObjPtr->color;
    ToObjPtr->bg_color = FromObjPtr->bg_color;
-   memcpy(FromObjPtr->color_str, ToObjPtr->color_str,
-         sizeof(FromObjPtr->color_str));
-   memcpy(FromObjPtr->bg_color_str, ToObjPtr->bg_color_str,
-         sizeof(FromObjPtr->bg_color_str));
+   memcpy(ToObjPtr->color_str, FromObjPtr->color_str,
+         sizeof(ToObjPtr->color_str));
+   memcpy(ToObjPtr->bg_color_str, FromObjPtr->bg_color_str,
+         sizeof(ToObjPtr->bg_color_str));
    ToObjPtr->id = objId++;
    ToObjPtr->dirty = FALSE;
    ToObjPtr->rotation = FromObjPtr->rotation;
@@ -241,6 +241,28 @@ void DupPolyObj(PolyPtr, ObjPtr)
       poly_ptr->sn = 0;
       poly_ptr->svlist = NULL;
    }
+   if (PolyPtr->ssn != 0 && PolyPtr->ssvlist != NULL) {
+      poly_ptr->ssn = num_pts = PolyPtr->ssn;
+      pv = (IntPoint*)malloc((num_pts+1)*sizeof(IntPoint));
+      if (pv == NULL) FailAllocMessage();
+      for (i=0; i < num_pts; i++) {
+         pv[i].x = PolyPtr->ssvlist[i].x;
+         pv[i].y = PolyPtr->ssvlist[i].y;
+      }
+      poly_ptr->ssvlist = pv;
+   } else {
+      poly_ptr->ssn = 0;
+      poly_ptr->ssvlist = NULL;
+   }
+   if (PolyPtr->ssmooth != NULL && PolyPtr->ssvlist != NULL) {
+      num_pts = PolyPtr->ssn;
+      poly_ptr->ssmooth = (char*)malloc((num_pts+1)*sizeof(char));
+      if (poly_ptr->ssmooth == NULL) FailAllocMessage();
+      memcpy(poly_ptr->ssmooth, PolyPtr->ssmooth, num_pts*sizeof(char));
+      poly_ptr->ssmooth[num_pts] = FALSE;
+   } else {
+      poly_ptr->ssmooth = NULL;
+   }
    if (PolyPtr->curved == LT_INTSPLINE && PolyPtr->intn != 0 &&
          PolyPtr->intvlist != NULL) {
       poly_ptr->intn = num_pts = PolyPtr->intn;
@@ -326,6 +348,28 @@ void DupPolygonObj(PolygonPtr, ObjPtr)
       polygon_ptr->sn = 0;
       polygon_ptr->svlist = NULL;
    }
+   if (PolygonPtr->ssn != 0 && PolygonPtr->ssvlist != NULL) {
+      polygon_ptr->ssn = num_pts = PolygonPtr->ssn;
+      pv = (IntPoint*)malloc((num_pts+1)*sizeof(IntPoint));
+      if (pv == NULL) FailAllocMessage();
+      for (i=0; i < num_pts; i++) {
+         pv[i].x = PolygonPtr->ssvlist[i].x;
+         pv[i].y = PolygonPtr->ssvlist[i].y;
+      }
+      polygon_ptr->ssvlist = pv;
+   } else {
+      polygon_ptr->ssn = 0;
+      polygon_ptr->ssvlist = NULL;
+   }
+   if (PolygonPtr->ssmooth != NULL && PolygonPtr->ssvlist != NULL) {
+      num_pts = PolygonPtr->ssn;
+      polygon_ptr->ssmooth = (char*)malloc((num_pts+1)*sizeof(char));
+      if (polygon_ptr->ssmooth == NULL) FailAllocMessage();
+      memcpy(polygon_ptr->ssmooth, PolygonPtr->ssmooth, num_pts*sizeof(char));
+      polygon_ptr->ssmooth[num_pts] = FALSE;
+   } else {
+      polygon_ptr->ssmooth = NULL;
+   }
    if (PolygonPtr->curved == LT_INTSPLINE && PolygonPtr->intn != 0 &&
          PolygonPtr->intvlist != NULL) {
       polygon_ptr->intn = num_pts = PolygonPtr->intn;
@@ -589,11 +633,9 @@ void DupXPmObj(XPmPtr, ObjPtr)
    struct XPmRec *XPmPtr;
    struct ObjRec *ObjPtr;
 {
-   register struct XPmRec *xpm_ptr;
-   register int j, i;
-   Pixmap pixmap, bitmap, cached_pixmap, cached_bitmap;
-   int w, h, ncolors, *pixels, len, chars_per_pixel, image_w, image_h;
-   char *color_char, **color_str;
+   register struct XPmRec *xpm_ptr=NULL;
+   Pixmap pixmap=None, bitmap=None, cached_pixmap=None, cached_bitmap=None;
+   int w=0, h=0, ncolors=0, image_w=0, image_h=0;
 
    xpm_ptr = (struct XPmRec *)malloc(sizeof(struct XPmRec));
    if (xpm_ptr == NULL) FailAllocMessage();
@@ -630,6 +672,25 @@ void DupXPmObj(XPmPtr, ObjPtr)
          xpm_ptr->filename = UtilStrDup(XPmPtr->filename);
          if (xpm_ptr->filename == NULL) FailAllocMessage();
       }
+   } else if (xpm_ptr->real_type == PPM_TRUE) {
+      xpm_ptr->ppm_data_compress = XPmPtr->ppm_data_compress;
+      if (xpm_ptr->ppm_data_compress == PPM_JPEG_COMPRESS ||
+            xpm_ptr->ppm_data_compress == PPM_DATA_DEFLATED) {
+         xpm_ptr->ppm_mask_size = XPmPtr->ppm_mask_size;
+         if (xpm_ptr->ppm_mask_size > 0) {
+            xpm_ptr->ppm_mask_data = (char*)malloc(xpm_ptr->ppm_mask_size);
+            if (xpm_ptr->ppm_mask_data == NULL) FailAllocMessage();
+            memcpy(xpm_ptr->ppm_mask_data, XPmPtr->ppm_mask_data,
+                  xpm_ptr->ppm_mask_size);
+         }
+         xpm_ptr->ppm_data_size = XPmPtr->ppm_data_size;
+         xpm_ptr->ppm_data = (char*)malloc(xpm_ptr->ppm_data_size);
+         if (xpm_ptr->ppm_data == NULL) FailAllocMessage();
+         memcpy(xpm_ptr->ppm_data, XPmPtr->ppm_data, xpm_ptr->ppm_data_size);
+         xpm_ptr->has_transparent_color = XPmPtr->has_transparent_color;
+         memcpy(xpm_ptr->transparent_color, XPmPtr->transparent_color,
+               sizeof(xpm_ptr->transparent_color));
+      }
    }
    if (ObjPtr->ctm != NULL) {
       memcpy(&xpm_ptr->cached_ctm, &XPmPtr->cached_ctm,
@@ -674,27 +735,36 @@ void DupXPmObj(XPmPtr, ObjPtr)
             w, h, 0, 0);
       xpm_ptr->cached_bitmap = cached_bitmap;
    }
-
-   chars_per_pixel = xpm_ptr->chars_per_pixel = XPmPtr->chars_per_pixel;
-   xpm_ptr->first_pixel_is_bg = XPmPtr->first_pixel_is_bg;
-
    ncolors = xpm_ptr->ncolors = XPmPtr->ncolors;
-   color_char = xpm_ptr->color_char =
-         (char*)malloc(ncolors*chars_per_pixel*sizeof(char));
-   color_str = xpm_ptr->color_str = (char**)malloc(ncolors*sizeof(char*));
-   pixels = xpm_ptr->pixels = (int*)malloc(ncolors*sizeof(int));
-   if (color_char==NULL || color_str==NULL || pixels==NULL) FailAllocMessage();
-   for (i=0; i < ncolors; i++) {
-      for (j=0; j < chars_per_pixel; j++) {
-         color_char[i*chars_per_pixel+j] =
-               XPmPtr->color_char[i*chars_per_pixel+j];
-      }
-      pixels[i] = XPmPtr->pixels[i];
 
-      len = strlen(XPmPtr->color_str[i]);
-      color_str[i] = (char*)malloc((len+1)*sizeof(char));
-      if (color_str[i] == NULL) FailAllocMessage();
-      strcpy(color_str[i], XPmPtr->color_str[i]);
+   if (xpm_ptr->real_type != PPM_TRUE) {
+      int i=0, *pixels=NULL, chars_per_pixel=0;
+      char *color_char=NULL, **color_str=NULL;
+
+      chars_per_pixel = xpm_ptr->chars_per_pixel = XPmPtr->chars_per_pixel;
+      xpm_ptr->first_pixel_is_bg = XPmPtr->first_pixel_is_bg;
+
+      color_char = xpm_ptr->color_char =
+            (char*)malloc(ncolors*chars_per_pixel*sizeof(char));
+      color_str = xpm_ptr->color_str = (char**)malloc(ncolors*sizeof(char*));
+      pixels = xpm_ptr->pixels = (int*)malloc(ncolors*sizeof(int));
+      if (color_char==NULL || color_str==NULL || pixels==NULL) {
+         FailAllocMessage();
+      }
+      for (i=0; i < ncolors; i++) {
+         int j=0, len=0;
+
+         for (j=0; j < chars_per_pixel; j++) {
+            color_char[i*chars_per_pixel+j] =
+                  XPmPtr->color_char[i*chars_per_pixel+j];
+         }
+         pixels[i] = XPmPtr->pixels[i];
+
+         len = strlen(XPmPtr->color_str[i]);
+         color_str[i] = (char*)malloc((len+1)*sizeof(char));
+         if (color_str[i] == NULL) FailAllocMessage();
+         strcpy(color_str[i], XPmPtr->color_str[i]);
+      }
    }
    xpm_ptr->red = xpm_ptr->green = xpm_ptr->blue = NULL;
    if (ObjPtr->ctm != NULL && XPmPtr->clip_mask != None) {
