@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2006, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2009, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/eps.c,v 1.4 2006/02/10 23:21:19 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/eps.c,v 1.8 2009/05/28 03:53:11 william Exp $
  */
 
 #define _INCLUDE_FROM_EPS_C_
@@ -585,13 +585,16 @@ int ConvertTiffToXbm(fp, tiff_offset, tiff_sz, xbm_fname, xbm_sz)
    if (MkTempFile(xbm_fname, xbm_sz, tmpDir, TOOL_NAME) == NULL) {
       return ErrorInConvertTiffToXbm(fp, NULL, NULL, NULL);
    }
-   if ((xbm_fp=fopen(xbm_fname, "w")) == NULL) {
-      return ErrorInConvertTiffToXbm(fp, NULL, NULL, NULL);
-   }
    if (!tiffToXbmCmdInitialized) {
       strcpy(tiffToXbmCmd, "tifftopnm %s | pgmtopbm | pbmtoxbm");
    }
    sprintf(cmd, tiffToXbmCmd, tiff_fname);
+   if (!FindProgramInPath(cmd, NULL, FALSE)) {
+      return ErrorInConvertTiffToXbm(fp, NULL, NULL, NULL);
+   }
+   if ((xbm_fp=fopen(xbm_fname, "w")) == NULL) {
+      return ErrorInConvertTiffToXbm(fp, NULL, NULL, NULL);
+   }
    sprintf(gszMsgBox, TgLoadCachedString(CSTID_EXECUTING_GIVEN_PROGRAM), cmd);
    SetStringStatus(gszMsgBox);
    if (mainDisplay != NULL) XSync(mainDisplay, False);
@@ -791,6 +794,11 @@ int ConvertPsToXbm(pszPsPath, pszXbmPath, file_buf_sz)
       FailAllocMessage();
       return FALSE;
    }
+   sprintf(pszCmd, psToXbmCmd, pszPsPath);
+   if (!FindProgramInPath(pszCmd, NULL, FALSE)) {
+      free(pszCmd);
+      return FALSE;
+   }
    if ((pFile=fopen(pszXbmPath,"w")) == NULL) {
       sprintf(gszMsgBox, TgLoadString(STID_CANNOT_OPEN_FILE_FOR_WRITING),
             pszXbmPath);
@@ -798,7 +806,6 @@ int ConvertPsToXbm(pszPsPath, pszXbmPath, file_buf_sz)
       free(pszCmd);
       return FALSE;
    }
-   sprintf(pszCmd, psToXbmCmd, pszPsPath);
    sprintf(gszMsgBox, TgLoadCachedString(CSTID_EXECUTING_GIVEN_PROGRAM),
          pszCmd);
    SetStringStatus(gszMsgBox);
@@ -1761,6 +1768,10 @@ struct ObjRec *CreateEPSObj(FileName, ImageW, ImageH, bitmap, image,
    obj_ptr->bbox.rby = obj_ptr->obbox.rby = h + drawOrigY;
    obj_ptr->type = OBJ_XBM;
    obj_ptr->color = colorIndex;
+   if (mainDisplay != NULL) {
+      UtilStrCpyN(obj_ptr->color_str, sizeof(obj_ptr->color_str),
+            colorMenuItems[colorIndex]);
+   }
    obj_ptr->id = objId++;
    obj_ptr->dirty = FALSE;
    obj_ptr->rotation = 0;
