@@ -5,7 +5,7 @@
  *
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 2001-2006, William Chia-Wei Cheng.
+ * Copyright (C) 2001-2009, William Chia-Wei Cheng.
  *
  * This file may be distributed under the terms of the Q Public License
  * as defined by Trolltech AS of Norway and appearing in the file
@@ -19,7 +19,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/ftp.c,v 1.4 2006/05/30 02:56:28 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/ftp.c,v 1.9 2009/10/03 04:26:51 william Exp $
  */
 
 #define _INCLUDE_FROM_FTP_C_
@@ -410,7 +410,7 @@ int FtpSendPortCmd(n_socket, pn_data_socket)
 
       addr_sz = sizeof(soc_address);
       rc = getsockname(n_socket, (struct sockaddr *)&soc_address,
-            (socklen_t *)(&addr_sz));
+            (void*)(&addr_sz));
       if (rc < 0) {
          close(*pn_data_socket);
          *pn_data_socket = (-1);
@@ -426,7 +426,7 @@ int FtpSendPortCmd(n_socket, pn_data_socket)
       }
       addr_sz = sizeof(soc_address);
       rc = getsockname(*pn_data_socket, (struct sockaddr *)&soc_address,
-            (socklen_t *)(&addr_sz));
+            (void*)(&addr_sz));
       if (rc < 0) {
          close(*pn_data_socket);
          *pn_data_socket = (-1);
@@ -654,7 +654,7 @@ int FtpGetContent(n_socket, data_socket, is_dir, ppsz_buf, pn_buf_sz)
    if (pn_buf_sz != NULL) *pn_buf_sz = 0;
    *ppsz_buf = NULL;
    accepted_socket = accept(data_socket, (struct sockaddr *)&soc_address,
-         (socklen_t *)(&soc_addrlen));
+         (void*)(&soc_addrlen));
    if (accepted_socket < 0) return TG_REMOTE_STATUS_HOST;
 
    ftpReadData = TRUE;
