@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/grid.c,v 1.7 2009/06/05 21:52:40 cvsps Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/grid.c,v 1.14 2009/10/21 23:20:08 cvsps Exp $
  */
 
 #define _INCLUDE_FROM_GRID_C_
@@ -88,6 +88,12 @@ int useRecentForDiagMouseMove=TRUE;
 int useRecentForDiagMouseMove=FALSE;
 #endif /* RESTRICTED_MOUSE_MOVE */
 
+int	defRightMargin=(ONE_INCH*TIK_PER_PIXEL);
+int	defRightMarginEnabled=INVALID;
+int	rightMargin=(ONE_INCH*TIK_PER_PIXEL);
+int	rightMarginEnabled=INVALID;
+int	rightMarginActive=TRUE;
+
 static
 void MyHDotLine(Win, gc, Y, XStart, XEnd)
    Window Win;
@@ -335,6 +341,68 @@ void DrawPageLines(Win, LtX, LtY, W, H)
    }
 }
 
+static
+void GetRightMarginSpec(buf)
+   char *buf;
+{
+   double dval=(double)0;
+
+   dval = ((double)rightMargin) * ((double)printMag) /
+         ((double)TIK_PER_PIXEL);
+   dval /= ((double)100);
+   PixelToCurrentUnit(buf, round(dval));
+}
+
+void ToggleRightMargin()
+{
+   switch (rightMarginEnabled) {
+   case INVALID: rightMarginEnabled = TRUE; break;
+   case FALSE: rightMarginEnabled = TRUE; break;
+   case TRUE: rightMarginEnabled = FALSE; break;
+   }
+   SetFileModified(TRUE);
+   RedrawHRulerWindow();
+
+   if (rightMarginEnabled == TRUE) {
+      char buf[80];
+
+      *buf = '\0';
+      GetRightMarginSpec(buf);
+      sprintf(gszMsgBox, TgLoadString(STID_RIGHT_MARGIN_IS), buf);
+      Msg(gszMsgBox);
+   } else {
+      sprintf(gszMsgBox, TgLoadString(STID_RIGHT_MARGIN_DISABLED));
+      Msg(gszMsgBox);
+   }
+}
+
+void SpecifyRightMargin()
+{
+   char buf[MAXSTRING+1], spec[MAXSTRING+1];
+   double dval=(double)0;
+
+   *gszMsgBox = '\0';
+   GetRightMarginSpec(gszMsgBox);
+
+   sprintf(buf, TgLoadString(STID_CURRENT_RIGHT_MARGIN_IS), gszMsgBox);
+   strcpy(gszMsgBox, TgLoadString(STID_ENTER_RIGHT_MARGIN));
+   *spec = '\0';
+   Dialog(gszMsgBox, buf, spec);
+   UtilTrimBlanks(spec);
+   if (*spec == '\0') return;
+   if (GetDimensionInDouble(spec, TRUE, &dval)) {
+      dval = dval * ((double)100) * ((double)TIK_PER_PIXEL) /
+            ((double)printMag);
+      rightMargin = round(dval);
+      RedrawHRulerWindow();
+      SetFileModified(TRUE);
+
+      GetRightMarginSpec(buf);
+      sprintf(gszMsgBox, TgLoadString(STID_RIGHT_MARGIN_IS), buf);
+      Msg(gszMsgBox);
+   }
+}
+
 void ToggleGridSystem()
 {
    int saved_orig_x, saved_orig_y;
@@ -1388,6 +1456,13 @@ int RefreshShowHideMenu(menu)
    /* ShowMode */
    ok &= TgSetMenuItemCheckById(menu, CMDID_TOGGLESHOWMODE, !noModeWindow);
 
+   /* ToggleRightMargin */
+   ok &= TgSetMenuItemCheckById(menu, CMDID_TOGGLERIGHTMARGIN,
+         rightMarginEnabled == TRUE);
+   /* SpecifyRightMargin */
+   ok &= TgEnableMenuItemById(menu, CMDID_SPECIFYRIGHTMARGIN,
+         rightMarginEnabled == TRUE);
+
    if (gstWBInfo.do_whiteboard) {
       /* ShowChat */
       ok &= TgEnableMenuItemById(menu, CMDID_TOGGLESHOWCHAT, TRUE);
@@ -1835,6 +1910,13 @@ int RefreshGridMenu(menu)
    ok &= TgSetMenuItemCheckById(menu, CMDID_TOGGLEGRIDSYSTEM,
          (gridSystem==METRIC_GRID));
 
+   /* ToggleRightMargin */
+   ok &= TgSetMenuItemCheckById(menu, CMDID_TOGGLERIGHTMARGIN,
+         rightMarginEnabled == TRUE);
+   /* SpecifyRightMargin */
+   ok &= TgEnableMenuItemById(menu, CMDID_SPECIFYRIGHTMARGIN,
+         rightMarginEnabled == TRUE);
+
    return ok;
 }
 
