@@ -5,7 +5,7 @@
  *
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2006, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2009, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -32,7 +32,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/http.c,v 1.3 2006/02/10 23:21:19 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/http.c,v 1.10 2009/09/28 22:39:48 william Exp $
  */
 
 #define _INCLUDE_FROM_HTTP_C_
@@ -145,7 +145,6 @@ static int gnDecode[] = {
    41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, -1, -1, -1, -1, -1
 };
 
-static
 void DoBase64Encode(buf_in, buf_len, return_buf_in)
    char *buf_in, *return_buf_in;
    int buf_len;
@@ -232,13 +231,11 @@ int Convert(buf4, count, buf3)
    return 0;
 }
 
-static
-void DoBase64Decode(buf, buf_sz, return_buf)
+int DoBase64Decode(buf, buf_sz, return_buf)
    char *buf, *return_buf;
    int buf_sz;
 {
-   int i=0, j=0;
-   int intbuf[5];
+   int i=0, j=0, bytes_decoded=0, intbuf[5];
 
    intbuf[4] = (-1);
    for (i=0, j=0; i < buf_sz; i++) {
@@ -260,9 +257,11 @@ void DoBase64Decode(buf, buf_sz, return_buf)
             *return_buf++ = (char)(out_buf[k] & 0x0ff);
          }
          j = 0;
+         bytes_decoded += bytes;
       }
    }
    *return_buf = '\0';
+   return bytes_decoded;
 }
 
 char *Base64Decode(buf)
@@ -275,7 +274,8 @@ char *Base64Decode(buf)
       FailAllocMessage();
       return NULL;
    }
-   DoBase64Decode(buf, len, return_buf);
+   (void)DoBase64Decode(buf, len, return_buf);
+
    return return_buf;
 }
 
@@ -463,8 +463,6 @@ void CleanUpHttp()
 
    UtilFree(gpszFakedReferer);
    gpszFakedReferer = NULL;
-
-   if (gnDecode) { }
 }
 
 void InitHttp()
@@ -845,7 +843,7 @@ int HttpDoWrite(n_socket, psz_path, psz_host, us_port)
       fprintf(stderr, "************************\n");
       fprintf(stderr, "* Begin Request Header *\n");
       fprintf(stderr, "************************\n");
-      fwrite(buf, sizeof(char), strlen(buf), stderr);
+      (void)fwrite(buf, sizeof(char), strlen(buf), stderr);
       fprintf(stderr, "************************\n");
       fprintf(stderr, "*  End Request Header  *\n");
       fprintf(stderr, "************************\n");
@@ -1306,10 +1304,15 @@ int ReadFromSocket(n_socket)
       status = select(nfds, &fdset, NULL, NULL, &timeout);
 #endif /* __hpux */
       if (status < 0) {
-         sprintf(gszMsgBox, TgLoadString(STID_FUNC_SELECT_SYS_CALL_FAILED),
-               "ReadFromSocket()"),
-         MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
-         break;
+         if (errno == EINTR) {
+            /* interrupted by a system call, not a problem */
+            CheckInterrupt(TRUE);
+         } else {
+            sprintf(gszMsgBox, TgLoadString(STID_FUNC_SELECT_SYS_CALL_FAILED),
+                  "ReadFromSocket()"),
+            MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+            break;
+         }
       } else if (status == 0) {
          /* spin the interrupt thing */
          CheckInterrupt(TRUE);
@@ -1454,7 +1457,7 @@ int HttpDoRead(n_socket, ppsz_buf, pn_buf_sz)
          /* debug, do not translate */
          if (!cmdLineDumpURL) {
             fprintf(stderr, "\n==========>>>\n");
-            fwrite(buf, sizeof(char), len, stderr);
+            (void)fwrite(buf, sizeof(char), len, stderr);
             fprintf(stderr, "\n<<<==========\n");
          }
       }
