@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 2001-2006, William Chia-Wei Cheng.
+ * Copyright (C) 2001-2009, William Chia-Wei Cheng.
  *
  * This file may be distributed under the terms of the Q Public License
  * as defined by Trolltech AS of Norway and appearing in the file
@@ -15,7 +15,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/mainloop.c,v 1.11 2006/06/06 00:20:53 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/mainloop.c,v 1.35 2009/10/13 05:10:19 cvsps Exp $
  */
 
 #define _INCLUDE_FROM_MAINLOOP_C_
@@ -80,6 +80,8 @@
 #include "strtbl.e"
 #include "tangram2.e"
 #include "text.e"
+#include "tgcwheel.e"
+#include "tginssym.e"
 #include "tidget.e"
 #include "util.e"
 #include "version.e"
@@ -239,7 +241,7 @@ void ExecWithFile(CmdName, FileName)
    /* do not translate -- program constants */
    sprintf(s, "xterm -bd red -e %s %s", CmdName, FileName);
    strcat(s, " &");
-   system(s);
+   (void)system(s);
 #else /* ~_BACKGROUND_DONT_FORK */
    int pid;
 
@@ -247,7 +249,7 @@ void ExecWithFile(CmdName, FileName)
    sprintf(s, "xterm -bd red -e %s %s", CmdName, FileName);
    pid = fork();
    if (pid == 0) {
-      system(s);
+      (void)system(s);
       exit(0);
    }
 #endif /* _BACKGROUND_DONT_FORK */
@@ -322,10 +324,9 @@ void CleanUp()
 
    TieLooseEnds();
 
-#ifdef _TANGRAM_II
-   CleanUpTangram2();
-#endif /* _TANGRAM_II */
-
+   if (cmdLineTgrm2) {
+      CleanUpTangram2();
+   }
    CleanUpMiniLines();
    CleanUpCutPaste();
    CleanUpProperties();
@@ -376,6 +377,8 @@ void CleanUp()
    CleanUpSpecial();
    CleanUpAttr();
    CleanUpTalkToSelfFiledes();
+   CleanUpInsertSymbol();
+   CleanUpColorWheel();
    CleanUpTidget();
 
    if (iconWindowCreated) {
@@ -504,7 +507,7 @@ int GzipFile(fname, gzipped_fname)
 {
    int rc=TRUE;
    FILE *pfp=NULL, *out_fp=NULL;
-   char *compress_cmd=gunzipCmd, *cmd=NULL;
+   char *compress_cmd=NULL, *cmd=NULL;
 
    if ((out_fp=fopen(gzipped_fname, "w")) == NULL) {
       return FailToOpenMessage(gzipped_fname, "w", NULL);
@@ -520,6 +523,12 @@ int GzipFile(fname, gzipped_fname)
    } else {
       sprintf(cmd, compress_cmd, fname);
    }
+   if (!FindProgramInPath(cmd, NULL, FALSE)) {
+      fclose(out_fp);
+      free(cmd);
+      RestoreStatusStrings();
+      return FALSE;
+   }
    sprintf(gszMsgBox, TgLoadCachedString(CSTID_EXECUTING_GIVEN_PROGRAM),
          gzipCmd);
    SetStringStatus(gszMsgBox);
@@ -596,6 +605,13 @@ char *GunzipFileIntoTemp(gzipped_fname)
    } else {
       sprintf(cmd, uncompress_cmd, gzipped_fname);
    }
+   if (!FindProgramInPath(cmd, NULL, FALSE)) {
+      fclose(out_fp);
+      unlink(tmp_fname);
+      free(cmd);
+      RestoreStatusStrings();
+      return NULL;
+   }
    sprintf(gszMsgBox, TgLoadCachedString(CSTID_EXECUTING_GIVEN_PROGRAM),
          gunzipCmd);
    if (PRTGIF) {
@@ -677,8 +693,7 @@ int FindXApplResDir(psz_srch_path, psz_target_dir, cb_target_dir)
 static
 void InitLocale()
 {
-   DIR *dirp=NULL;
-   char package[80], *c_ptr=NULL, locale_dir[MAXPATHLENGTH];
+   char *c_ptr=NULL;
 
    c_ptr = MySetLocale(LC_ALL, "");
    if (c_ptr == NULL) {
@@ -716,6 +731,13 @@ void InitLocale()
     * From this point on, setlocale() should NEVER be called again!
     */
    setlocale(LC_NUMERIC, "C");
+}
+
+static
+void InitLocale2()
+{
+   DIR *dirp=NULL;
+   char package[80], *c_ptr=NULL, locale_dir[MAXPATHLENGTH];
 
    *locale_dir = '\0';
    if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"LocaleDir")) != NULL) {
@@ -1070,6 +1092,16 @@ int JustLoadAFile(pszFile, pnNeedToCheckAutoExec, nAttemptToMerge, pnAbortMerge)
    return TRUE;
 }
 
+static
+void SigchldHandler(sig)
+   int sig;
+{
+   int status;
+
+   /* wait for all the zombie processes to terminate */
+   while(waitpid(-1, &status, WNOHANG) > 0);
+}
+
 int JustInit(pszFile, pnNeedToCheckAutoExec)
    char *pszFile;
    int *pnNeedToCheckAutoExec;
@@ -1123,6 +1155,14 @@ int JustInit(pszFile, pnNeedToCheckAutoExec)
 #endif /* USE_XAPPLRESDIR */
    useXlib = TRUE;
 #endif /* USE_XT_INITIALIZE */
+
+#ifndef _NO_LOCALE_SUPPORT
+   MySetLocale(LC_ALL, "C");
+#ifdef ENABLE_NLS
+   InitLocale();
+#endif /* ENABLE_NLS */
+#endif /* ~_NO_LOCALE_SUPPORT */
+
    if (useXlib) {
       cannot_opendisplay = ((mainDisplay=XOpenDisplay(displayName)) == 0);
    } else {
@@ -1168,22 +1208,23 @@ int JustInit(pszFile, pnNeedToCheckAutoExec)
    signal(SIGBUS, EmergencySave);
 #endif /* ~linux */
    signal(SIGSEGV, EmergencySave);
-   signal(SIGCHLD, SIG_IGN);
+   signal(SIGCHLD, SigchldHandler);
 
 #ifndef _NO_LOCALE_SUPPORT
-   MySetLocale(LC_ALL, "C");
 #ifdef ENABLE_NLS
-   InitLocale();
+   InitLocale2();
 #endif /* ENABLE_NLS */
 #endif /* ~_NO_LOCALE_SUPPORT */
 
+   InitWhiteBoard();
+
    Setup();
    InitExtraWinInfo();
    InitLastKeyOrBtnEvInfo();
    quitDraw = FALSE;
    prTgifFoundColorInfo = FALSE;
 
-   InitWhiteBoard();
+   InitWhiteBoard2();
    if (pszFile == NULL) {
       /* Only gets here if PRTGIF of UrlDump() */
       DelAllPages();
@@ -1201,7 +1242,6 @@ int JustInit(pszFile, pnNeedToCheckAutoExec)
             "Whiteboard initialized too many times in JustInit()", NULL);
 #endif /* _TGIF_DBG */
       if (gstWBInfo.join_session_in_progress) {
-         WBCompleteJoin();
          allow_open = FALSE;
       }
       initialized = TRUE;
@@ -1342,26 +1382,113 @@ int JustInit(pszFile, pnNeedToCheckAutoExec)
 
 void HandleDataInMBuff()
 {
-#ifdef _TGIF_WB2
    if (gstWBInfo.do_whiteboard) {
       TryHandleWBInputData();
    }
-#endif /* _TGIF_WB2 */
 }
 
 static
 int GetAnXEvent(pXEvent)
    XEvent *pXEvent;
 {
-#ifdef _TGIF_WB2
-   int rc=TRUE;
+   if (gstWBInfo.do_whiteboard) {
+      int rc=TRUE;
 
-   gstWBInfo.BlockRemoteCmdDepth--;
+      gstWBInfo.BlockRemoteCmdDepth--;
 #ifdef DEBUG0
-   fprintf(stderr,"BlockRemoteCmdDepth (GetAnXEvent): %d\n",
-         gstWBInfo.BlockRemoteCmdDepth);
+      fprintf(stderr,"BlockRemoteCmdDepth (GetAnXEvent): %d\n",
+            gstWBInfo.BlockRemoteCmdDepth);
 #endif /* DEBUG0 */
-   if (XPending(mainDisplay)) {
+      if (XPending(mainDisplay)) {
+         if (gnInputMethod != TGIM_NONE && tgIMExpectNextEvent(mainDisplay,
+               drawWindow)) {
+            if (!tgIMHandleNextEvent(mainDisplay, drawWindow, pXEvent)) {
+            }
+         } else {
+            XNextEvent(mainDisplay, pXEvent);
+         }
+      } else {
+         struct timeval timeout;
+         fd_set fdset;
+         int select_width=XConnectionNumber(mainDisplay)+1, status=0, nfds=0;
+
+         nfds = max(select_width, (int)(talkToSelfFiledes[0]+1));
+
+         timeout.tv_sec = 15;
+         timeout.tv_usec = 0;
+
+         FD_ZERO(&fdset);
+         FD_SET(select_width-1, &fdset); /* X file descriptor */
+         FD_SET(talkToSelfFiledes[0], &fdset); /* pipe */
+#ifdef __hpux
+         status = select(nfds, (int*)&fdset, NULL, NULL, &timeout);
+#else /* !__hpux */
+         status = select(nfds, &fdset, NULL, NULL, &timeout);
+#endif /* __hpux */
+         if (status < 0) {
+            if (errno == EINTR) {
+               /* interrupted by a system call, not a problem */
+            } else {
+               sprintf(gszMsgBox,
+                     TgLoadString(STID_FUNC_SELECT_SYS_CALL_FAILED),
+                     "GetAnXEvent()"),
+               fprintf(stderr, "%s\n", gszMsgBox);
+            }
+            rc = FALSE;
+         } else if (status == 0) {
+            /* timeout */
+            rc = FALSE;
+         } else if (status == 1) {
+            if (FD_ISSET(talkToSelfFiledes[0], &fdset)) {
+               char buf[2];
+
+               if (read(talkToSelfFiledes[0], buf, 1) != 1) {
+                  sprintf(gszMsgBox,
+                        TgLoadString(STID_READ_FROM_SELF_PIPE_FAIL), 1,
+                        TOOL_NAME),
+                  fprintf(stderr, "%s\n", gszMsgBox);
+               } else {
+                  if (*buf == 'a') {
+                     SendCommandToSelf(CMDID_DATA_IN_MBUFF, 0);
+                  } else if (*buf == 'c') {
+                     /*
+                      * Get current tgwb state and establish
+                      *         a TCP conection to send the data.
+                      */
+                     HandleNewUserRequest();
+                  } else {
+                     sprintf(gszMsgBox,
+                           TgLoadString(STID_UNRECOG_BYTE_FROM_SELF_PIPE), *buf,
+                           TOOL_NAME),
+                     fprintf(stderr, "%s\n", gszMsgBox);
+                  }
+               }
+               rc = FALSE;
+            } else if (FD_ISSET(select_width-1, &fdset)) {
+               if (gnInputMethod != TGIM_NONE && tgIMExpectNextEvent(mainDisplay,
+                     drawWindow)) {
+                  if (!tgIMHandleNextEvent(mainDisplay, drawWindow, pXEvent)) {
+                  }
+               } else {
+                  XNextEvent(mainDisplay, pXEvent);
+               }
+            } else {
+               rc = FALSE;
+            }
+         } else {
+            sprintf(gszMsgBox, TgLoadString(STID_FUNC_INVALID_RC_FOR_SELECT),
+                  "GetAnXEvent()", status);
+            fprintf(stderr, "%s\n", gszMsgBox);
+            rc = FALSE;
+         }
+      }
+      gstWBInfo.BlockRemoteCmdDepth++;
+#ifdef DEBUG0
+      fprintf(stderr,"BlockRemoteCmdDepth (GetAnXEvent): %d\n",
+            gstWBInfo.BlockRemoteCmdDepth);
+#endif /* DEBUG0 */
+      if (!rc) return FALSE;
+   } else {
       if (gnInputMethod != TGIM_NONE && tgIMExpectNextEvent(mainDisplay,
             drawWindow)) {
          if (!tgIMHandleNextEvent(mainDisplay, drawWindow, pXEvent)) {
@@ -1369,94 +1496,7 @@ int GetAnXEvent(pXEvent)
       } else {
          XNextEvent(mainDisplay, pXEvent);
       }
-   } else {
-      struct timeval timeout;
-      fd_set fdset;
-      int select_width=XConnectionNumber(mainDisplay)+1, status=0, nfds=0;
-
-      nfds = max(select_width, (int)(talkToSelfFiledes[0]+1));
-
-      timeout.tv_sec = 15;
-      timeout.tv_usec = 0;
-
-      FD_ZERO(&fdset);
-      FD_SET(select_width-1, &fdset); /* X file descriptor */
-      FD_SET(talkToSelfFiledes[0], &fdset); /* pipe */
-#ifdef __hpux
-      status = select(nfds, (int*)&fdset, NULL, NULL, &timeout);
-#else /* !__hpux */
-      status = select(nfds, &fdset, NULL, NULL, &timeout);
-#endif /* __hpux */
-      if (status < 0) {
-         sprintf(gszMsgBox, TgLoadString(STID_FUNC_SELECT_SYS_CALL_FAILED),
-               "GetAnXEvent()"),
-         fprintf(stderr, "%s\n", gszMsgBox);
-         rc = FALSE;
-      } else if (status == 0) {
-         /* timeout */
-         rc = FALSE;
-      } else if (status == 1) {
-         if (FD_ISSET(talkToSelfFiledes[0], &fdset)) {
-            char buf[2];
-
-            if (read(talkToSelfFiledes[0], buf, 1) != 1) {
-               sprintf(gszMsgBox,
-                     TgLoadString(STID_READ_FROM_SELF_PIPE_FAIL), 1,
-                     TOOL_NAME),
-               fprintf(stderr, "%s\n", gszMsgBox);
-            } else {
-               if (*buf == 'a') {
-                  SendCommandToSelf(CMDID_DATA_IN_MBUFF, 0);
-               } else if (*buf == 'c') {
-                  /*
-                   * Get current tgwb state and establish
-                   *         a TCP conection to send the data.
-                   */
-                  HandleNewUserRequest();
-               } else {
-                  sprintf(gszMsgBox,
-                        TgLoadString(STID_UNRECOG_BYTE_FROM_SELF_PIPE), *buf,
-                        TOOL_NAME),
-                  fprintf(stderr, "%s\n", gszMsgBox);
-               }
-            }
-            rc = FALSE;
-         } else if (FD_ISSET(select_width-1, &fdset)) {
-            if (gnInputMethod != TGIM_NONE && tgIMExpectNextEvent(mainDisplay,
-                  drawWindow)) {
-               if (!tgIMHandleNextEvent(mainDisplay, drawWindow, pXEvent)) {
-               }
-            } else {
-               XNextEvent(mainDisplay, pXEvent);
-            }
-         } else {
-            rc = FALSE;
-         }
-      } else {
-         sprintf(gszMsgBox, TgLoadString(STID_FUNC_INVALID_RC_FOR_SELECT),
-               "GetAnXEvent()", status);
-         fprintf(stderr, "%s\n", gszMsgBox);
-         rc = FALSE;
-      }
    }
-   gstWBInfo.BlockRemoteCmdDepth++;
-#ifdef DEBUG0
-   fprintf(stderr,"BlockRemoteCmdDepth (GetAnXEvent): %d\n",
-         gstWBInfo.BlockRemoteCmdDepth);
-#endif /* DEBUG0 */
-   if (!rc) return FALSE;
-#endif /* _TGIF_WB2 */
-
-#if !defined(_TGIF_WB) && !defined(_TGIF_WB2)
-   if (gnInputMethod != TGIM_NONE && tgIMExpectNextEvent(mainDisplay,
-         drawWindow)) {
-      if (!tgIMHandleNextEvent(mainDisplay, drawWindow, pXEvent)) {
-      }
-   } else {
-      XNextEvent(mainDisplay, pXEvent);
-   }
-#endif /* !defined(_TGIF_WB) && !defined(_TGIF_WB2) */
-
    return TRUE;
 }
 
@@ -1464,7 +1504,7 @@ int TryProcessAnXEvent(pnNeedToCheckAutoExec)
    int *pnNeedToCheckAutoExec;
 {
    XEvent input;
-   int rc=0;
+   int rc=0, drawtext_cursorshown_keypress_processed=FALSE;
    TidgetInfo *handling_pti=NULL;
 
    exitNormally = FALSE;
@@ -1520,22 +1560,21 @@ int TryProcessAnXEvent(pnNeedToCheckAutoExec)
          return rc;
       }
    } else if (input.xany.window == drawWindow) {
-#ifdef _TGIF_WB2
       if (input.type == ButtonPress) {
          if (gstWBInfo.do_whiteboard) {
             gstWBInfo.BlockRemoteCmdDepth++;
          }
       }
-#endif /* _TGIF_WB2 */
+      if (curChoice == DRAWTEXT && textCursorShown && input.type == KeyPress) {
+          drawtext_cursorshown_keypress_processed = TRUE;
+      }
       rc = DrawingEventHandler(&input);
-#ifdef _TGIF_WB2
       if (input.type == ButtonPress) {
          if (gstWBInfo.do_whiteboard) {
             gstWBInfo.BlockRemoteCmdDepth--;
             TryHandleWBInputData();
          }
       }
-#endif /* _TGIF_WB2 */
       if (rc != INVALID) {
          return rc;
       }
@@ -1570,6 +1609,9 @@ int TryProcessAnXEvent(pnNeedToCheckAutoExec)
       StatusEventHandler(&input);
    } else if (chatWindow != None && IsChatWindowEvent(&input, &handling_pti)) {
       ChatEventHandler(&input, handling_pti);
+      if (curChoice == DRAWTEXT && textCursorShown && input.type == KeyPress) {
+          drawtext_cursorshown_keypress_processed = TRUE;
+      }
    } else if (modeWindow != None && input.xany.window == modeWindow) {
       if ((rc=ModeEventHandler(&input)) != INVALID) {
          return rc;
@@ -1603,6 +1645,15 @@ int TryProcessAnXEvent(pnNeedToCheckAutoExec)
          }
       }
    }
+#ifndef _NO_ENTER_TEXT_EVERYWHERE
+   if (curChoice == DRAWTEXT && textCursorShown && input.type == KeyPress &&
+         !drawtext_cursorshown_keypress_processed) {
+      rc = DrawingEventHandler(&input);
+      if (rc != INVALID) {
+         return rc;
+      }
+   }
+#endif /* ~_NO_ENTER_TEXT_EVERYWHERE */
    return 0;
 }
 
