@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/mainloop.c,v 1.34 2009/10/02 20:14:03 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/mainloop.c,v 1.35 2009/10/13 05:10:19 cvsps Exp $
  */
 
 #define _INCLUDE_FROM_MAINLOOP_C_
@@ -94,6 +94,7 @@
 #include "tangram2.e"
 #include "text.e"
 #include "tgcwheel.e"
+#include "tginssym.e"
 #include "tidget.e"
 #include "util.e"
 #include "version.e"
@@ -389,6 +390,7 @@ void CleanUp()
    CleanUpSpecial();
    CleanUpAttr();
    CleanUpTalkToSelfFiledes();
+   CleanUpInsertSymbol();
    CleanUpColorWheel();
    CleanUpTidget();
 
