@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/menuinfo.c,v 1.45 2009/07/20 16:58:03 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/menuinfo.c,v 1.50 2009/10/22 03:44:01 cvsps Exp $
  */
 
 #define _INCLUDE_FROM_MENUINFO_C_
@@ -276,10 +276,15 @@ static TgMenuItemInfo fontStyleMenuItemInfo[] = {
    { N_("Right"), "Alt+Cntrl+r", N_("Right justified text"), NULL,
      CMDID_CHANGEALLSELFONTJUST },
    { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
-   { N_("No Underlined"), NULL, N_("No underlined text"), NULL,
+   { N_("No Underline"), NULL, N_("No underlined text"), NULL,
      CMDID_CHANGEALLSELFONTUNDERLINE },
-   { N_("Underlined"), NULL, N_("Underlined text"), NULL,
+   { N_("Underline"), NULL, N_("Underlined text"), NULL,
      CMDID_CHANGEALLSELFONTUNDERLINE },
+   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
+   { N_("No Overline"), NULL, N_("No overlined text"), NULL,
+     CMDID_CHANGEALLSELFONTOVERLINE },
+   { N_("Overline"), NULL, N_("Overlined text"), NULL,
+     CMDID_CHANGEALLSELFONTOVERLINE },
    { NULL, NULL, NULL, NULL, INVALID }
 };
 TgMenuInfo fontStyleMenuInfo={ TGMUTYPE_TEXT, fontStyleMenuItemInfo,
@@ -879,6 +884,10 @@ static TgMenuItemInfo textMenuItemInfo[] = {
    { N_("Insert Center Subscript"), NULL,
      N_("Insert subscript below the text cursor"), NULL,
      CMDID_INSERTCENTERSUBSCRIPT },
+   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
+   { N_("Insert Symbol..."), NULL,
+     N_("Insert a symbol from the Symbol Font"), NULL,
+     CMDID_INSERTSYMBOL },
 #ifdef _TGIF_DBG
    { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
    { N_("Insert Hex/Octal Character..."), NULL,
@@ -1257,6 +1266,13 @@ static TgMenuItemInfo showHideMenuItemInfo[] = {
    { N_("Set Measure Unit..."), NULL, N_("Set the unit of measurements"), NULL,
      CMDID_SETMEASUREUNIT },
    { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
+   { N_("Enable Right Margin"), NULL,
+     N_("Toggle between using (simple) right margin or not"), NULL,
+     CMDID_TOGGLERIGHTMARGIN },
+   { N_("Specify Right Margin..."), NULL,
+     N_("Specify the (simple) right margin"), NULL,
+     CMDID_SPECIFYRIGHTMARGIN },
+   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
    { N_("Show Menubar"), NULL, N_("Show menubar window"), NULL,
      CMDID_TOGGLESHOWMENUBAR },
    { N_("Show Choice"), NULL, N_("Show choice and message window"), NULL,
@@ -1782,6 +1798,12 @@ static TgMenuItemInfo gridMenuItemInfo[] = {
    { N_("Metric Grid"), "Alt+Cntrl+-",
      N_("Toggle between English and Metric grid systems"), NULL,
      CMDID_TOGGLEGRIDSYSTEM },
+   { N_("Enable Right Margin"), NULL,
+     N_("Toggle between using (simple) right margin or not"), NULL,
+     CMDID_TOGGLERIGHTMARGIN },
+   { N_("Specify Right Margin..."), NULL,
+     N_("Specify the (simple) right margin"), NULL,
+     CMDID_SPECIFYRIGHTMARGIN },
    { NULL, NULL, NULL, NULL, INVALID }
 };
 TgMenuInfo gridMenuInfo={ TGMUTYPE_TEXT, gridMenuItemInfo, CreateGridMenu };
