@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 2001-2006, William Chia-Wei Cheng.
+ * Copyright (C) 2001-2009, William Chia-Wei Cheng.
  *
  * This file may be distributed under the terms of the Q Public License
  * as defined by Trolltech AS of Norway and appearing in the file
@@ -15,7 +15,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/select.c,v 1.5 2006/02/10 23:21:21 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/select.c,v 1.24 2009/09/28 17:34:30 william Exp $
  */
 
 #define _INCLUDE_FROM_SELECT_C_
@@ -32,6 +32,7 @@
 #include "dialog.e"
 #include "drawing.e"
 #include "dup.e"
+#include "edit.e"
 #include "exec.e"
 #include "file.e"
 #include "font.e"
@@ -76,6 +77,19 @@ int	numObjLocked=0;
 struct SelRec	*topSel=NULL, *botSel=NULL;
 struct VSelRec	*topVSel=NULL, *botVSel=NULL;
 
+int GetObjCurved(ObjPtr)
+   struct ObjRec *ObjPtr;
+{
+   switch (ObjPtr->type) {
+   case OBJ_POLY: return ObjPtr->detail.p->curved;
+   case OBJ_POLYGON: return ObjPtr->detail.g->curved;
+   }
+#ifdef _TGIF_DBG /* debug, do not translate */
+   TgAssert(FALSE, "Unexpected object type in GetObjCurved()", NULL);
+#endif /* _TGIF_DBG */
+   return 0;
+}
+
 struct SelRec *SelectThisObject(ObjPtr)
    struct ObjRec *ObjPtr;
 {
@@ -93,14 +107,23 @@ int CountSelectedVertices()
    int count = 0;
 
    for (vsel_ptr=topVSel; vsel_ptr != NULL; vsel_ptr=vsel_ptr->next) {
-      int i, n=vsel_ptr->n;
+      int curved=GetObjCurved(vsel_ptr->obj);
 
-      count += n;
-      if (vsel_ptr->obj->type == OBJ_POLYGON) {
-         for (i = 0; i < n; i++) {
-            if (vsel_ptr->v_index[i] == 0) {
-               count--;
-               break;
+      if (curved == LT_STRUCT_SPLINE) {
+         /* can select at most one vertex for LT_STRUCT_SPLINE */
+         count++;
+      } else {
+         int n=vsel_ptr->n;
+
+         count += n;
+         if (vsel_ptr->obj->type == OBJ_POLYGON) {
+            int i=0;
+
+            for (i=0; i < n; i++) {
+               if (vsel_ptr->v_index[i] == 0) {
+                  count--;
+                  break;
+               }
             }
          }
       }
@@ -382,13 +405,14 @@ struct ObjRec *FindAVertex(XOff, YOff, VIndex, AbsX, AbsY)
    int XOff, YOff, *VIndex, *AbsX, *AbsY;
    /* XOff and YOff are screen offsets */
 {
-   register struct ObjRec *obj_ptr;
-   struct PolyRec *poly_ptr;
-   struct PolygonRec *polygon_ptr;
    struct SelRec *sel_ptr;
 
    for (sel_ptr=topSel; sel_ptr != NULL; sel_ptr=sel_ptr->next) {
-      obj_ptr = sel_ptr->obj;
+      struct ObjRec *obj_ptr=sel_ptr->obj;
+      struct PolyRec *poly_ptr=NULL;
+      struct PolygonRec *polygon_ptr=NULL;
+      int n=0;
+      IntPoint *vs=NULL;
 
       if (obj_ptr->type != OBJ_POLY && obj_ptr->type != OBJ_POLYGON) continue;
       if (!(XOff >= OFFSET_X(obj_ptr->bbox.ltx)-3 &&
@@ -401,18 +425,22 @@ struct ObjRec *FindAVertex(XOff, YOff, VIndex, AbsX, AbsY)
       switch (obj_ptr->type) {
       case OBJ_POLY:
          poly_ptr = obj_ptr->detail.p;
-         if (PtInPolyMark(obj_ptr, XOff, YOff, poly_ptr->n,
-               poly_ptr->vlist, VIndex)) {
+         if (poly_ptr->curved == LT_STRUCT_SPLINE) {
+            n = poly_ptr->ssn;
+            vs = poly_ptr->ssvlist;
+         } else {
+            n = poly_ptr->n;
+            vs = poly_ptr->vlist;
+         }
+         if (PtInPolyMark(obj_ptr, XOff, YOff, n, vs, VIndex)) {
             if (obj_ptr->ctm == NULL) {
-               *AbsX = poly_ptr->vlist[*VIndex].x;
-               *AbsY = poly_ptr->vlist[*VIndex].y;
+               *AbsX = vs[*VIndex].x;
+               *AbsY = vs[*VIndex].y;
             } else {
                int x, y;
 
-               TransformPointThroughCTM(
-                     poly_ptr->vlist[*VIndex].x-obj_ptr->x,
-                     poly_ptr->vlist[*VIndex].y-obj_ptr->y,
-                     obj_ptr->ctm, &x, &y);
+               TransformPointThroughCTM( vs[*VIndex].x-obj_ptr->x,
+                     vs[*VIndex].y-obj_ptr->y, obj_ptr->ctm, &x, &y);
                *AbsX = x+obj_ptr->x;
                *AbsY = y+obj_ptr->y;
             }
@@ -421,18 +449,22 @@ struct ObjRec *FindAVertex(XOff, YOff, VIndex, AbsX, AbsY)
          break;
       case OBJ_POLYGON:
          polygon_ptr = obj_ptr->detail.g;
-         if (PtInPolyMark(obj_ptr, XOff, YOff, polygon_ptr->n,
-               polygon_ptr->vlist, VIndex)) {
+         if (polygon_ptr->curved == LT_STRUCT_SPLINE) {
+            n = polygon_ptr->ssn;
+            vs = polygon_ptr->ssvlist;
+         } else {
+            n = polygon_ptr->n;
+            vs = polygon_ptr->vlist;
+         }
+         if (PtInPolyMark(obj_ptr, XOff, YOff, n, vs, VIndex)) {
             if (obj_ptr->ctm == NULL) {
-               *AbsX = polygon_ptr->vlist[*VIndex].x;
-               *AbsY = polygon_ptr->vlist[*VIndex].y;
+               *AbsX = vs[*VIndex].x;
+               *AbsY = vs[*VIndex].y;
             } else {
                int x, y;
 
-               TransformPointThroughCTM(
-                     polygon_ptr->vlist[*VIndex].x-obj_ptr->x,
-                     polygon_ptr->vlist[*VIndex].y-obj_ptr->y,
-                     obj_ptr->ctm, &x, &y);
+               TransformPointThroughCTM(vs[*VIndex].x-obj_ptr->x,
+                     vs[*VIndex].y-obj_ptr->y, obj_ptr->ctm, &x, &y);
                *AbsX = x+obj_ptr->x;
                *AbsY = y+obj_ptr->y;
             }
@@ -867,6 +899,7 @@ struct VSelRec *SelectOneVertex(XOff, YOff)
    }
    topVSel = (struct VSelRec *)malloc(sizeof(struct VSelRec));
    if (topVSel == NULL) FailAllocMessage();
+   memset(topVSel, 0, sizeof(struct VSelRec));
    topVSel->obj = obj_ptr;
    topVSel->max_v = 10;
    topVSel->v_index = (int*)malloc(10*sizeof(int));
@@ -923,33 +956,48 @@ struct SelRec *SelectOneObj(XOff, YOff, ppInnerObj)
 }
 
 static
-int FindVertices(X1, Y1, X2, Y2, TopVSel, BotVSel)
-   int X1, Y1, X2, Y2;
+int FindVertices(X1, Y1, X2, Y2, TopVSel, BotVSel, pn_struct_spline_msg)
+   int X1, Y1, X2, Y2, *pn_struct_spline_msg;
    struct VSelRec **TopVSel, **BotVSel;
    /* X1, Y1, X2, Y2 are absolute coordinates */
 {
-   register struct ObjRec *obj_ptr;
-   register struct VSelRec *vsel_ptr;
-   register int i;
-   struct SelRec *sel_ptr;
+   struct SelRec *sel_ptr=NULL;
    struct BBRec bbox;
-   IntPoint *v;
-   int n, count, max_count, j;
 
    *TopVSel = *BotVSel = NULL;
 
    bbox.ltx = X1; bbox.lty = Y1;
    bbox.rbx = X2; bbox.rby = Y2;
    for (sel_ptr=botSel; sel_ptr != NULL; sel_ptr=sel_ptr->prev) {
-      obj_ptr = sel_ptr->obj;
+      int i=0, n=0, count=0, max_count=0, j=0, curved=(-1);
+      IntPoint *v=NULL;
+      struct ObjRec *obj_ptr=sel_ptr->obj;
 
       if (obj_ptr->type != OBJ_POLY && obj_ptr->type != OBJ_POLYGON) continue;
       if (!BBoxIntersect(bbox, obj_ptr->bbox)) continue;
 
-      v = (obj_ptr->type==OBJ_POLY) ? obj_ptr->detail.p->vlist :
-            obj_ptr->detail.g->vlist;
-      n = (obj_ptr->type==OBJ_POLY) ? obj_ptr->detail.p->n :
-            obj_ptr->detail.g->n;
+      switch (obj_ptr->type) {
+      case OBJ_POLY:
+         curved = obj_ptr->detail.p->curved;
+         if (curved == LT_STRUCT_SPLINE) {
+            n = obj_ptr->detail.p->ssn;
+            v = obj_ptr->detail.p->ssvlist;
+         } else {
+            n = obj_ptr->detail.p->n;
+            v = obj_ptr->detail.p->vlist;
+         }
+         break;
+      case OBJ_POLYGON:
+         curved = obj_ptr->detail.g->curved;
+         if (curved == LT_STRUCT_SPLINE) {
+            n = obj_ptr->detail.g->ssn;
+            v = obj_ptr->detail.g->ssvlist;
+         } else {
+            n = obj_ptr->detail.g->n;
+            v = obj_ptr->detail.g->vlist;
+         }
+         break;
+      }
       if (obj_ptr->ctm == NULL) {
          for (i = 0, count = 0; i < n; i++) {
             if (v[i].x >= X1 && v[i].x <= X2 && v[i].y >= Y1 && v[i].y <= Y2) {
@@ -968,10 +1016,19 @@ int FindVertices(X1, Y1, X2, Y2, TopVSel, BotVSel)
             }
          }
       }
-
       if (count != 0) {
+         struct VSelRec *vsel_ptr=NULL;
+
+         if (curved == LT_STRUCT_SPLINE && count > 1) {
+            if (!(*pn_struct_spline_msg)) {
+               *pn_struct_spline_msg = TRUE;
+               Msg(TgLoadString(STID_ONE_V_STRUCT_SPLINE_IN_V_MODE));
+            }
+            count = 1;
+         }
          vsel_ptr = (struct VSelRec *)malloc(sizeof(struct VSelRec));
          if (vsel_ptr == NULL) FailAllocMessage();
+         memset(vsel_ptr, 0, sizeof(struct VSelRec));
          vsel_ptr->obj = obj_ptr;
          vsel_ptr->next = *TopVSel;
          vsel_ptr->prev = NULL;
@@ -1187,37 +1244,37 @@ struct ObjRec *PtInSelected(XOff, YOff)
 }
 
 static
-void ToggleVertexSelection(ObjPtr, VIndex, AbsX, AbsY)
+void ToggleVertexSelection(ObjPtr, VIndex, AbsX, AbsY, pn_struct_spline_msg)
    struct ObjRec *ObjPtr;
-   int VIndex, AbsX, AbsY;
+   int VIndex, AbsX, AbsY, *pn_struct_spline_msg;
 {
-   int i, j, n;
-   struct VSelRec *vsel_ptr;
+   int i, j, n=0, curved=(-1), last_polygon_vertex=FALSE;
+   struct VSelRec *vsel_ptr=NULL;
+   char *smooth=NULL;
 
-   if (!(ObjPtr->type==OBJ_POLYGON && ObjPtr->detail.g->n-1==VIndex)) {
-      char *smooth=NULL;
-      int curved=(-1);
-
-      switch (ObjPtr->type) {
-      case OBJ_POLY:
+   switch (ObjPtr->type) {
+   case OBJ_POLY:
+      curved = ObjPtr->detail.p->curved;
+      if (curved == LT_STRUCT_SPLINE) {
+         smooth = ObjPtr->detail.p->ssmooth;
+      } else {
          smooth = ObjPtr->detail.p->smooth;
-         curved = ObjPtr->detail.p->curved;
-         break;
-      case OBJ_POLYGON:
-         smooth = ObjPtr->detail.g->smooth;
-         curved = ObjPtr->detail.g->curved;
-         break;
       }
-      if (curved != LT_INTSPLINE && curved != (-1) && smooth != NULL) {
-         if (smooth[VIndex]) {
-            MARKO(drawWindow, revDefaultGC, OFFSET_X(AbsX), OFFSET_Y(AbsY));
-         } else {
-            MARK(drawWindow, revDefaultGC, OFFSET_X(AbsX), OFFSET_Y(AbsY));
+      break;
+   case OBJ_POLYGON:
+      curved = ObjPtr->detail.g->curved;
+      if (curved == LT_STRUCT_SPLINE) {
+         smooth = ObjPtr->detail.g->ssmooth;
+         if (VIndex == ObjPtr->detail.g->ssn-1) {
+            last_polygon_vertex = TRUE;
          }
       } else {
-         MARK(drawWindow, revDefaultGC, OFFSET_X(AbsX), OFFSET_Y(AbsY));
+         smooth = ObjPtr->detail.g->smooth;
+         if (VIndex == ObjPtr->detail.g->n-1) {
+            last_polygon_vertex = TRUE;
+         }
       }
-      MARKV(drawWindow, revDefaultGC, OFFSET_X(AbsX), OFFSET_Y(AbsY));
+      break;
    }
    if (VertexAlreadySelected(ObjPtr, VIndex, &vsel_ptr)) {
       /* de-select a vertex */
@@ -1256,8 +1313,10 @@ void ToggleVertexSelection(ObjPtr, VIndex, AbsX, AbsY)
       }
    } else {
       if (vsel_ptr == NULL) {
+         /* a new object */
          vsel_ptr = (struct VSelRec *)malloc(sizeof(struct VSelRec));
          if (vsel_ptr == NULL) FailAllocMessage();
+         memset(vsel_ptr, 0, sizeof(struct VSelRec));
          vsel_ptr->obj = ObjPtr;
          n = vsel_ptr->n = 1;
          vsel_ptr->max_v = 10;
@@ -1276,22 +1335,50 @@ void ToggleVertexSelection(ObjPtr, VIndex, AbsX, AbsY)
          }
          topVSel = vsel_ptr;
       } else {
-         n = ++(vsel_ptr->n);
-         if (n > vsel_ptr->max_v) {
-            int max_v;
+         if (!last_polygon_vertex && curved == LT_STRUCT_SPLINE) {
+            if (!(*pn_struct_spline_msg)) {
+               *pn_struct_spline_msg = TRUE;
+               Msg(TgLoadString(STID_ONE_V_STRUCT_SPLINE_IN_V_MODE));
+            }
+            return;
+         } else {
+            n = ++(vsel_ptr->n);
+            if (n > vsel_ptr->max_v) {
+               int max_v;
 
-            vsel_ptr->max_v += 10;
-            max_v = vsel_ptr->max_v;
-            vsel_ptr->v_index = (int*)realloc(vsel_ptr->v_index,
-                  sizeof(int)*max_v);
-            vsel_ptr->x = (int*)realloc(vsel_ptr->x, sizeof(int)*max_v);
-            vsel_ptr->y = (int*)realloc(vsel_ptr->y, sizeof(int)*max_v);
+               vsel_ptr->max_v += 10;
+               max_v = vsel_ptr->max_v;
+               vsel_ptr->v_index = (int*)realloc(vsel_ptr->v_index,
+                     sizeof(int)*max_v);
+               vsel_ptr->x = (int*)realloc(vsel_ptr->x, sizeof(int)*max_v);
+               vsel_ptr->y = (int*)realloc(vsel_ptr->y, sizeof(int)*max_v);
+            }
          }
       }
       vsel_ptr->v_index[n-1] = VIndex;
       vsel_ptr->x[n-1] = AbsX;
       vsel_ptr->y[n-1] = AbsY;
    }
+   if (!last_polygon_vertex) {
+      if (curved != LT_INTSPLINE && curved != (-1) && smooth != NULL) {
+         if (smooth[VIndex]) {
+            if (curved == LT_STRUCT_SPLINE) {
+               MARKHO(drawWindow, revDefaultGC, OFFSET_X(AbsX), OFFSET_Y(AbsY));
+            } else {
+               MARKO(drawWindow, revDefaultGC, OFFSET_X(AbsX), OFFSET_Y(AbsY));
+            }
+         } else {
+            if (curved == LT_STRUCT_SPLINE) {
+               MARKHR(drawWindow, revDefaultGC, OFFSET_X(AbsX), OFFSET_Y(AbsY));
+            } else {
+               MARK(drawWindow, revDefaultGC, OFFSET_X(AbsX), OFFSET_Y(AbsY));
+            }
+         }
+      } else {
+         MARK(drawWindow, revDefaultGC, OFFSET_X(AbsX), OFFSET_Y(AbsY));
+      }
+      MARKV(drawWindow, revDefaultGC, OFFSET_X(AbsX), OFFSET_Y(AbsY));
+   }
 }
 
 static
@@ -1328,15 +1415,15 @@ void ContinueSel(XOff, YOff, ShiftKeyDown)
    int XOff, YOff, ShiftKeyDown;
    /* XOff and YOff are screen offsets, and they are not on grid */
 {
-   register int i;
-   int end_x, end_y, v_index, new_end_x, new_end_y;
+   int i, end_x, end_y, v_index, new_end_x, new_end_y;
    int done=FALSE, ltx, lty, rbx, rby, dx, dy, x, y;
+   int struct_spline_msg=FALSE;
    char buf[80], w_buf[80], h_buf[80], x_buf[80], y_buf[80];
    XEvent input, ev;
-   XMotionEvent *motion_ev;
-   struct SelRec *sel_ptr, *top_sel_ptr, *bot_sel_ptr;
-   struct VSelRec *vsel_ptr, *top_vsel_ptr, *bot_vsel_ptr;
-   struct ObjRec *obj_ptr, *owner_obj;
+   XMotionEvent *motion_ev=NULL;
+   struct SelRec *sel_ptr=NULL, *top_sel_ptr=NULL, *bot_sel_ptr=NULL;
+   struct VSelRec *vsel_ptr=NULL, *top_vsel_ptr=NULL, *bot_vsel_ptr=NULL;
+   struct ObjRec *obj_ptr=NULL, *owner_obj=NULL;
 
    end_x = XOff;
    end_y = YOff; 
@@ -1411,9 +1498,16 @@ void ContinueSel(XOff, YOff, ShiftKeyDown)
          } else if (ShiftKeyDown) {
             obj_ptr = FindAVertex(XOff, YOff, &v_index, &x, &y);
             if (obj_ptr != NULL) {
-               ToggleVertexSelection(obj_ptr, v_index, x, y);
+               ToggleVertexSelection(obj_ptr, v_index, x, y, 
+                     &struct_spline_msg);
                if (obj_ptr->type==OBJ_POLYGON && v_index==0) {
-                  ToggleVertexSelection(obj_ptr,obj_ptr->detail.g->n-1,x,y);
+                  if (obj_ptr->detail.g->curved == LT_STRUCT_SPLINE) {
+                     ToggleVertexSelection(obj_ptr,obj_ptr->detail.g->ssn-1,
+                           x, y, &struct_spline_msg);
+                  } else {
+                     ToggleVertexSelection(obj_ptr,obj_ptr->detail.g->n-1,
+                           x, y, &struct_spline_msg);
+                  }
                }
                UpdSelBBox();
             }
@@ -1435,14 +1529,14 @@ void ContinueSel(XOff, YOff, ShiftKeyDown)
             }
          } else if (ShiftKeyDown) {
             if (FindVertices(ABS_X(ltx), ABS_Y(lty), ABS_X(rbx), ABS_Y(rby),
-                  &top_vsel_ptr, &bot_vsel_ptr)) {
+                  &top_vsel_ptr, &bot_vsel_ptr, &struct_spline_msg)) {
                struct VSelRec *next_vsel;
 
                for (vsel_ptr=top_vsel_ptr; vsel_ptr!=NULL; vsel_ptr=next_vsel) {
                   obj_ptr = vsel_ptr->obj;
                   for (i = 0; i < vsel_ptr->n; i++) {
                      ToggleVertexSelection(obj_ptr, vsel_ptr->v_index[i],
-                           vsel_ptr->x[i], vsel_ptr->y[i]);
+                           vsel_ptr->x[i], vsel_ptr->y[i], &struct_spline_msg);
                   }
                   next_vsel = vsel_ptr->next;
                   free(vsel_ptr->v_index);
@@ -1456,7 +1550,7 @@ void ContinueSel(XOff, YOff, ShiftKeyDown)
             HighLightReverse();
             JustRemoveAllVSel();
             if (FindVertices(ABS_X(ltx), ABS_Y(lty), ABS_X(rbx), ABS_Y(rby),
-                  &top_vsel_ptr, &bot_vsel_ptr)) {
+                  &top_vsel_ptr, &bot_vsel_ptr, &struct_spline_msg)) {
                topVSel = top_vsel_ptr;
                botVSel = bot_vsel_ptr;
                UpdSelBBox();
@@ -1745,6 +1839,10 @@ void Select(input)
             RotateShearSel(grid_x, grid_y, sel_ptr->obj, corner);
             return;
          }
+         if (!autoRotatePivot && PtInRotatePivot(mouse_x, mouse_y)) {
+            ContinueMoveRotatePivot(grid_x, grid_y);
+            return;
+         }
       }
       ContinueSel(mouse_x, mouse_y, FALSE);
       justDupped = FALSE;
@@ -1832,10 +1930,74 @@ struct AttrRec *FindAttrWithName(ObjPtr, AttrName, pp_top_owner)
       if (pp_top_owner != NULL) *pp_top_owner = tgifObj;
       return FindFileAttrWithName(dot_ptr);
    }
-   if ((obj_ptr=FindObjWithName(botObj, ObjPtr, AttrName, FALSE,
-         FALSE, NULL, pp_top_owner)) == NULL) {
-      *dot_ptr++ = '.';
-      return NULL;
+   if (AttrName[0] == '#' && AttrName[1] == '#') {
+      int obj_page_num=0;
+      char *bang_ptr=strchr(&AttrName[2], '!');
+
+      if (bang_ptr == NULL) {
+         *dot_ptr = '.';
+         return NULL;
+      }
+      *bang_ptr = '\0';
+      if (sscanf(&AttrName[2], "%d", &obj_page_num) != 1) {
+         *bang_ptr = '!';
+         *dot_ptr = '.';
+         return NULL;
+      }
+      if (obj_page_num < 1 || obj_page_num > lastPageNum) {
+         sprintf(gszMsgBox, TgLoadString(STID_INVALID_PAGE_NUM),
+               &AttrName[2]),
+         MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+         *bang_ptr = '!';
+         *dot_ptr = '.';
+         return NULL;
+      }
+      *bang_ptr = '!';
+      if (curPageNum == obj_page_num) {
+         if ((obj_ptr=FindObjWithName(botObj, ObjPtr, bang_ptr, FALSE,
+               FALSE, NULL, pp_top_owner)) == NULL) {
+            *dot_ptr = '.';
+            return NULL;
+         }
+      } else {
+         struct PageRec *page_ptr=NULL;
+         int page_num=0;
+
+         for (page_ptr=firstPage, page_num=1; page_ptr != NULL;
+               page_ptr=page_ptr->next, page_num++) {
+            if (page_num == obj_page_num) {
+               struct PageRec *saved_cur_page_ptr=curPage;
+               int saved_cur_page_num=curPageNum;
+
+               curPageNum = page_num;
+               curPage = page_ptr;
+               topObj = curPage->top;
+               botObj = curPage->bot;
+               obj_ptr = FindObjWithName(botObj, NULL, bang_ptr,
+                     FALSE, FALSE, NULL, NULL);
+               curPageNum = saved_cur_page_num;
+               curPage = saved_cur_page_ptr;
+               topObj = curPage->top;
+               botObj = curPage->bot;
+
+               if (obj_ptr == NULL) {
+                  *dot_ptr = '.';
+                  return NULL;
+               }
+               break;
+            }
+         }
+         if (page_ptr == NULL) {
+            *dot_ptr++ = '.';
+            return NULL;
+         }
+      }
+   } else {
+      if ((obj_ptr=FindObjWithName(botObj, ObjPtr, AttrName, FALSE,
+            FALSE, NULL, pp_top_owner)) == NULL) {
+         *dot_ptr++ = '.';
+         return NULL;
+      }
    }
    *dot_ptr++ = '.';
    return FindObjAttrWithName(obj_ptr, dot_ptr);
@@ -2326,17 +2488,50 @@ void MoveSelToBot()
    RecordCmd(CMD_REPLACE, NULL, topSel, botSel, numObjSelected);
 }
 
+static
+void DeleteObjectInsteadOfVertex(obj_ptr)
+   struct ObjRec *obj_ptr;
+{
+   struct SelRec *sel_ptr=NULL, *saved_top_sel=NULL, *saved_bot_sel=NULL;
+
+   for (sel_ptr=botSel; sel_ptr!=NULL; sel_ptr=sel_ptr->prev) {
+      if (sel_ptr->obj == obj_ptr) {
+         break;
+      }
+   }
+   if (sel_ptr->prev == NULL) {
+      topSel = sel_ptr->next;
+   } else {
+      sel_ptr->prev->next = sel_ptr->next;
+   }
+   if (sel_ptr->next == NULL) {
+      botSel = sel_ptr->prev;
+   } else {
+      sel_ptr->next->prev = sel_ptr->prev;
+   }
+   saved_top_sel = topSel;
+   saved_bot_sel = botSel;
+   topSel = botSel = sel_ptr;
+   sel_ptr->next = sel_ptr->prev = NULL;
+   DelObj(obj_ptr);
+   topSel = saved_top_sel;
+   botSel = saved_bot_sel;
+
+   free(sel_ptr);
+   ChangeReplaceOneCmdToDeleteCmd();
+}
+
 void DelAllSelObj()
 {
-   register struct ObjRec *obj_ptr;
-   register struct SelRec *sel_ptr;
-   register int j, i;
-   struct VSelRec *vsel_ptr;
-   IntPoint *v=NULL;
+   struct ObjRec *obj_ptr=NULL;
+   struct SelRec *sel_ptr=NULL;
+   int j, i;
+   struct VSelRec *vsel_ptr=NULL;
+   IntPoint *vlist=NULL;
    struct PolyRec *poly_ptr=NULL;
    struct PolygonRec *polygon_ptr=NULL;
    int n=0;
-   short *mark;
+   short *mark=NULL;
 
    if (topSel==NULL && topVSel==NULL) {
       MsgBox(TgLoadCachedString(CSTID_NO_OBJ_SELECTED), TOOL_NAME, INFO_MB);
@@ -2346,27 +2541,39 @@ void DelAllSelObj()
    if (curChoice == VERTEXMODE) {
       StartCompositeCmd();
       for (vsel_ptr=botVSel; vsel_ptr!=NULL; vsel_ptr=vsel_ptr->prev) {
-         int delete_it=FALSE, extra_vertex=FALSE;
-         char *smooth=NULL;
+         int delete_it=FALSE, extra_vertex=FALSE, curved=(-1), ssn=0;
+         char *smooth=NULL, *ssmooth=NULL;
+         IntPoint *ssvlist=NULL;
 
          obj_ptr = vsel_ptr->obj;
 
-         PrepareToReplaceAnObj(obj_ptr);
          switch (obj_ptr->type) {
          case OBJ_POLY:
             poly_ptr = obj_ptr->detail.p;
-            v = poly_ptr->vlist;
-            n = poly_ptr->n;
-            smooth = poly_ptr->smooth;
-
+            curved = poly_ptr->curved;
+            if (curved == LT_STRUCT_SPLINE) {
+               ssvlist = poly_ptr->ssvlist;
+               ssn = poly_ptr->ssn;
+               ssmooth = poly_ptr->ssmooth;
+            } else {
+               vlist = poly_ptr->vlist;
+               n = poly_ptr->n;
+               smooth = poly_ptr->smooth;
+            }
             if (vsel_ptr->n >= n-1) delete_it = TRUE;
             break;
          case OBJ_POLYGON:
             polygon_ptr = obj_ptr->detail.g;
-            v = polygon_ptr->vlist;
-            n = polygon_ptr->n;
-            smooth = polygon_ptr->smooth;
-
+            curved = polygon_ptr->curved;
+            if (curved == LT_STRUCT_SPLINE) {
+               ssvlist = polygon_ptr->ssvlist;
+               ssn = polygon_ptr->ssn;
+               ssmooth = polygon_ptr->ssmooth;
+            } else {
+               vlist = polygon_ptr->vlist;
+               n = polygon_ptr->n;
+               smooth = polygon_ptr->smooth;
+            }
             for (j=0; j < vsel_ptr->n; j++) {
                if (vsel_ptr->v_index[j] == 0) {
                   extra_vertex = TRUE;
@@ -2379,93 +2586,89 @@ void DelAllSelObj()
             }
             break;
          }
-         if (delete_it) {
-            struct SelRec *saved_top_sel, *saved_bot_sel;
-
-            for (sel_ptr=botSel; sel_ptr!=NULL; sel_ptr=sel_ptr->prev) {
-               if (sel_ptr->obj == obj_ptr) {
-                  break;
+         if (curved == LT_STRUCT_SPLINE) {
+            PrepareToReplaceAnObj(obj_ptr);
+            if (DeleteStructuredSplinePoint(vsel_ptr->v_index[0], poly_ptr,
+                  polygon_ptr)) {
+               AdjObjSplineVs(obj_ptr);
+               if (poly_ptr != NULL) {
+                  UpdPolyBBox(obj_ptr, poly_ptr->n, poly_ptr->vlist);
+               } else if (polygon_ptr != NULL) {
+                  UpdPolyBBox(obj_ptr, polygon_ptr->n,
+                        polygon_ptr->vlist);
                }
-            }
-            if (sel_ptr->prev == NULL) {
-               topSel = sel_ptr->next;
+               AdjObjBBox(obj_ptr);
+               RecordReplaceAnObj(obj_ptr);
             } else {
-               sel_ptr->prev->next = sel_ptr->next;
+               /* delete the object */
+               DeleteObjectInsteadOfVertex(obj_ptr);
             }
-            if (sel_ptr->next == NULL) {
-               botSel = sel_ptr->prev;
-            } else {
-               sel_ptr->next->prev = sel_ptr->prev;
-            }
-            saved_top_sel = topSel;
-            saved_bot_sel = botSel;
-            topSel = botSel = sel_ptr;
-            sel_ptr->next = sel_ptr->prev = NULL;
-            DelObj(obj_ptr);
-            topSel = saved_top_sel;
-            botSel = saved_bot_sel;
-
-            free(sel_ptr);
-            ChangeReplaceOneCmdToDeleteCmd();
          } else {
-            mark = (short *)malloc(n*sizeof(short));
-            if (mark == NULL) FailAllocMessage();
-            for (j=0; j < n; j++) mark[j] = FALSE;
-            for (j=0; j < vsel_ptr->n; j++) mark[vsel_ptr->v_index[j]] = TRUE;
-
-            switch (obj_ptr->type) {
-            case OBJ_POLY:
-               for (i=n-1; i >= 0; i--) {
-                  if (mark[i]) {
-                     for (j=i+1; j < n; j++) {
-                        v[j-1] = v[j];
-                        if (smooth != NULL) smooth[j-1] = smooth[j];
+            PrepareToReplaceAnObj(obj_ptr);
+            if (delete_it) {
+               /* delete the object */
+               DeleteObjectInsteadOfVertex(obj_ptr);
+            } else {
+               mark = (short *)malloc(n*sizeof(short));
+               if (mark == NULL) FailAllocMessage();
+               for (j=0; j < n; j++) mark[j] = FALSE;
+               for (j=0; j < vsel_ptr->n; j++) {
+                  mark[vsel_ptr->v_index[j]] = TRUE;
+               }
+               switch (obj_ptr->type) {
+               case OBJ_POLY:
+                  for (i=n-1; i >= 0; i--) {
+                     if (mark[i]) {
+                        for (j=i+1; j < n; j++) {
+                           vlist[j-1] = vlist[j];
+                           if (smooth != NULL) smooth[j-1] = smooth[j];
+                        }
+                        if (smooth != NULL) {
+                           if (i == 0) {
+                              smooth[0] = FALSE;
+                           } else if (i == n-1) {
+                              smooth[n-2] = FALSE;
+                           }
+                        }
+                        n--;
                      }
-                     if (smooth != NULL) {
-                        if (i == 0) {
-                           smooth[0] = FALSE;
-                        } else if (i == n-1) {
-                           smooth[n-2] = FALSE;
+                  }
+                  poly_ptr->n -= vsel_ptr->n;
+                  AdjObjSplineVs(obj_ptr);
+                  if (poly_ptr->curved != LT_INTSPLINE) {
+                     UpdPolyBBox(obj_ptr, poly_ptr->n, poly_ptr->vlist);
+                  } else {
+                     UpdPolyBBox(obj_ptr, poly_ptr->intn, poly_ptr->intvlist);
+                  }
+                  break;
+               case OBJ_POLYGON:
+                  for (i=n-2; i >= 0; i--) {
+                     if (mark[i]) {
+                        for (j=i+1; j < n; j++) {
+                           vlist[j-1] = vlist[j];
+                           if (smooth != NULL) smooth[j-1] = smooth[j];
+                        }
+                        n--;
+                        if (smooth != NULL && i == 0) {
+                           smooth[n-1] = smooth[0];
                         }
                      }
-                     n--;
                   }
-               }
-               poly_ptr->n -= vsel_ptr->n;
-               AdjObjSplineVs(obj_ptr);
-               if (poly_ptr->curved != LT_INTSPLINE) {
-                  UpdPolyBBox(obj_ptr, poly_ptr->n, poly_ptr->vlist);
-               } else {
-                  UpdPolyBBox(obj_ptr, poly_ptr->intn, poly_ptr->intvlist);
-               }
-               break;
-            case OBJ_POLYGON:
-               for (i=n-2; i >= 0; i--) {
-                  if (mark[i]) {
-                     for (j=i+1; j < n; j++) {
-                        v[j-1] = v[j];
-                        if (smooth != NULL) smooth[j-1] = smooth[j];
-                     }
-                     n--;
-                     if (smooth != NULL && i == 0) {
-                        smooth[n-1] = smooth[0];
-                     }
+                  polygon_ptr->n -= vsel_ptr->n;
+                  if (extra_vertex) vlist[polygon_ptr->n++] = vlist[0];
+                  AdjObjSplineVs(obj_ptr);
+                  if (polygon_ptr->curved != LT_INTSPLINE) {
+                     UpdPolyBBox(obj_ptr, polygon_ptr->n, polygon_ptr->vlist);
+                  } else {
+                     UpdPolyBBox(obj_ptr, polygon_ptr->intn,
+                           polygon_ptr->intvlist);
                   }
+                  break;
                }
-               polygon_ptr->n -= vsel_ptr->n;
-               if (extra_vertex) v[polygon_ptr->n++] = v[0];
-               AdjObjSplineVs(obj_ptr);
-               if (polygon_ptr->curved != LT_INTSPLINE) {
-                  UpdPolyBBox(obj_ptr, polygon_ptr->n, polygon_ptr->vlist);
-               } else {
-                  UpdPolyBBox(obj_ptr, polygon_ptr->intn,
-                        polygon_ptr->intvlist);
-               }
-               break;
+               free(mark);
+               AdjObjBBox(obj_ptr);
+               RecordReplaceAnObj(obj_ptr);
             }
-            free(mark);
-            AdjObjBBox(obj_ptr);
-            RecordReplaceAnObj(obj_ptr);
          }
       }
       EndCompositeCmd();
