@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 2001-2003, William Chia-Wei Cheng.
+ * Copyright (C) 2001-2009, William Chia-Wei Cheng.
  *
  * This file may be distributed under the terms of the Q Public License
  * as defined by Trolltech AS of Norway and appearing in the file
@@ -15,7 +15,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/setup.c,v 1.11 2005/01/15 00:23:23 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/setup.c,v 1.71 2009/11/08 00:25:03 cvsps Exp $
  */
 
 #define _INCLUDE_FROM_SETUP_C_
@@ -77,6 +77,8 @@
 #include "tangram2.e"
 #include "tdgtlist.e"
 #include "text.e"
+#include "tgcwheel.e"
+#include "tginssym.e"
 #include "tidget.e"
 #include "util.e"
 #include "version.e"
@@ -211,6 +213,8 @@ int	zoomedIn = FALSE;
 struct BBRec	drawWinBBox;
 
 int	colorDisplay = FALSE;
+int	fullTrueColorMode = FALSE;
+int	noAntiAlias = TRUE;
 int	threeDLook = FALSE;
 int	fileModified = FALSE;
 int	objId = 0;
@@ -262,11 +266,14 @@ int	canvasWindowOnly=FALSE;
 int	talkToSelfFiledesInitialized=FALSE;
 int	talkToSelfFiledes[2];
 
+Atom	compoundTextAtom=(Atom)0;
+Atom	textAtom=(Atom)0;
+Atom	utf8StringAtom=(Atom)0;
+Atom	tgifProtocolAtom=(Atom)0;
+Atom	tmpSelectionAtom=(Atom)0;
+
 static Atom	wmDeleteWindowAtom=(Atom)0;
-static Atom	utf8StringAtom=(Atom)0;
-static Atom	compoundTextAtom=(Atom)0;
-static Atom	textAtom=(Atom)0;
-static Atom	stringAtom=(Atom)0;
+static Atom	targetsAtom=(Atom)0;
 
 static int	baseScrollBarW=SCROLLBAR_W;
 
@@ -372,6 +379,7 @@ void InitWinSizes()
       pageDummyWindowW = PAGE_DUMMY_WINDOW_W + windowPadding;
    }
    pageWindowW = drawWinW+rulerW-hSBarW-pageDummyWindowW;
+   if (pageWindowW <= 0) pageWindowW = 1;
    colorWindowH = drawWinH+rulerW-DEF_VSBAR_H-colorDummyWindowH;
    vSBarH = drawWinH+rulerW+(brdrW<<1);
    titleWindowW = (noModeWindow ? 0 : modeWindowW+(brdrW<<1)) +
@@ -411,6 +419,7 @@ void InverseInitWinSizes()
          (noModeWindow ? 0 : modeWindowW+(brdrW<<1));
    modeWindowH = drawWinH+rulerW+(brdrW<<2);
    pageWindowW = titleWindowW-scrollBarW-hSBarW-pageDummyWindowW-(brdrW<<2);
+   if (pageWindowW <= 0) pageWindowW = 1;
    colorWindowH = drawWinH+rulerW-DEF_VSBAR_H-colorDummyWindowH;
    if (colorLayers) vSBarH = DEF_VSBAR_H;
    CalcMenubarWindowHeight();
@@ -459,8 +468,12 @@ void DoReconfigure()
    }
    if (!noChoiceWindow) {
       XMoveWindow(mainDisplay, choiceWindow, 0, cur_y);
-      XMoveResizeWindow(mainDisplay, msgWindow, choiceWindowW+(brdrW<<1), cur_y,
-            msgWindowW, msgWindowH);
+      if (msgWindowW > 0) {
+         XMoveResizeWindow(mainDisplay, msgWindow, choiceWindowW+(brdrW<<1),
+               cur_y, msgWindowW, msgWindowH);
+      } else {
+         XMoveWindow(mainDisplay, msgWindow, choiceWindowW+(brdrW<<1), cur_y);
+      }
       cur_y += msgWindowH + (brdrW<<1);
    }
    if (!noModeWindow) {
@@ -496,8 +509,13 @@ void DoReconfigure()
       draw_win_y_offset = cur_y;
    }
    cur_y += drawWinH + (brdrW<<1);
-   XMoveResizeWindow(mainDisplay, pageWindow, 0, cur_y, pageWindowW,
-         scrollBarW+(brdrW<<1));
+   if (pageWindowW > 0) {
+      XMoveResizeWindow(mainDisplay, pageWindow, 0, cur_y, pageWindowW,
+            scrollBarW+(brdrW<<1));
+      MakeCurPageTabVisible();
+   } else {
+      XMoveWindow(mainDisplay, pageWindow, 0, cur_y);
+   }
    XMoveWindow(mainDisplay, pageDummyWindow, pageWindowW, cur_y);
    XMoveWindow(mainDisplay, hSBarWindow,
          pageWindowW+pageDummyWindowW+(brdrW<<1), cur_y);
@@ -616,6 +634,7 @@ void Reconfigure(Forced)
       titleWindowW = initDrawWinW + rulerW + modeWindowW + scrollBarW +
             (brdrW*6);
    }
+   if (pageWindowW <= 0) pageWindowW = 1;
    menubarWindowW = titleWindowW;
    chatWindowW = titleWindowW;
    if (!noChatWindow) SetChatWindowGeom();
@@ -655,23 +674,61 @@ int TgIsCmdEvent(pXEv)
    return FALSE;
 }
 
-#ifdef ICCCM_TEST
 static
+void HandleSelectionClear(input)
+   XEvent *input;
+{
+   Window win=(Window)0;
+   Atom selection=(Atom)0;
+   Time time;
+
+   win = input->xselectionclear.window;
+   selection = input->xselectionclear.selection;
+   time = input->xselectionclear.time;
+
+#ifdef NOT_DEFINED
+   {
+      static int id=0;
+      char *atom_name=NULL;
+
+      printf("\n%1d: SelectionClear.\n", ++id);
+
+      atom_name = XGetAtomName(mainDisplay, selection);
+      printf("%1d: Selection name is '%s'.\n", id, atom_name);
+      XFree(atom_name);
+   }
+#endif /* NOT_DEFINED */
+   if (startSelectionOwnershipTimeValid) {
+      endSelectionOwnershipTime = time;
+      endSelectionOwnershipTimeValid = TRUE;
+   }
+   ClearSelection();
+}
+
 void HandleSelectionRequest(input)
    XEvent *input;
 {
-   static int id=0;
-   Window      requestor=input->xselectionrequest.requestor;
-   Atom        property=(Atom)0, target=(Atom)0, selection=(Atom)0;
+   Window requestor=(Window)0;
+   Atom property=(Atom)0, target=(Atom)0, selection=(Atom)0;
+   Time time=(Time)0;
+   XEvent sent_ev;
 
+   requestor = input->xselectionrequest.requestor;
    selection = input->xselectionrequest.selection;
    target = input->xselectionrequest.target;
    property = input->xselectionrequest.property;
+   time = input->xselectionrequest.time;
 
-   printf("\n%1d: SelectionRequest.\n", ++id);
+#ifdef NOT_DEFINED
+   {
+      /*
+       * After tgif does a Copy, tgif will get SelectionRequest at
+       *         a regular interval
+       */
+      static int id=0;
+      char *atom_name=NULL;
 
-   if (property != None) {
-      char     *atom_name=NULL;
+      printf("\n%1d: SelectionRequest.\n", ++id);
 
       atom_name = XGetAtomName(mainDisplay, property);
       printf("%1d: Property name is '%s'.\n", id, atom_name);
@@ -680,36 +737,62 @@ void HandleSelectionRequest(input)
       atom_name = XGetAtomName(mainDisplay, target);
       printf("%1d: Target name is '%s'.\n", id, atom_name);
       XFree(atom_name);
-   } else {
-      property = target;
    }
-   if (input->xselectionrequest.owner == mainWindow) {
-      printf("%1d: owner == mainWindow!\n", id);
-   } else {
-      XEvent      sent_ev;
+#endif /* NOT_DEFINED */
+   memset(&sent_ev, 0, sizeof(XEvent));
 
-      memset(&sent_ev, 0, sizeof(XEvent));
+   sent_ev.xselection.type = SelectionNotify;
+   sent_ev.xselection.display = mainDisplay;
+   sent_ev.xselection.requestor = requestor;
+   sent_ev.xselection.selection = selection;
+   sent_ev.xselection.time = input->xselectionrequest.time;
+   sent_ev.xselection.target = target;
+   sent_ev.xselection.property = None;
 
-      sent_ev.xselectionrequest.type = SelectionNotify;
-      sent_ev.xselectionrequest.display = mainDisplay;
-      sent_ev.xselectionrequest.requestor = mainWindow;
-      sent_ev.xselectionrequest.selection = selection;
-      sent_ev.xselectionrequest.time = input->xselectionrequest.time;
-      sent_ev.xselectionrequest.target = target;
-      sent_ev.xselectionrequest.property = property;
+   if (target == targetsAtom) {
+      /*
+       * The selection manager is asking tgif what can you convert the
+       *         object copied into (i.e., what are the possobiel targets)?
+       */
+      Atom targets[5];
 
-      sent_ev.xselectionrequest.send_event = True;
-      sent_ev.xselectionrequest.owner = requestor;
+      targets[0] = XA_STRING;
+      targets[1] = compoundTextAtom;
+      targets[2] = tgifProtocolAtom;
+      targets[3] = textAtom;
+      targets[4] = utf8StringAtom;
+      XChangeProperty(mainDisplay, requestor, property, target, 32,
+            PropModeReplace, (unsigned char *)targets, 5);
+      sent_ev.xselection.property = property;
+   } else if (target == XA_STRING || target == textAtom) {
+      XChangeProperty(mainDisplay, requestor, property, target, 8,
+            PropModeReplace, (unsigned char *)(dsCutBuffer.s), dsCutBuffer.sz);
+      sent_ev.xselection.property = property;
+   } else if (target == utf8StringAtom) {
+      XChangeProperty(mainDisplay, requestor, property, target, 8,
+            PropModeReplace, (unsigned char *)(dsCutBuffer.s), dsCutBuffer.sz);
+      sent_ev.xselection.property = property;
+   } else if (target == tgifProtocolAtom) {
+      XChangeProperty(mainDisplay, requestor, property, target, 8,
+            PropModeReplace, (unsigned char *)(dsCutBuffer.s), dsCutBuffer.sz);
+      sent_ev.xselection.property = property;
+   } else if (target == compoundTextAtom) {
+      XTextProperty text_property;
 
-/*       if (target == utf8StringAtom || target == compoundTextAtom ||
-            target == textAtom || target == stringAtom) {
+      if (cutBufferIsUTF8) {
+         Xutf8TextListToTextProperty(mainDisplay, &(dsCutBuffer.s), 1,
+               XCompoundTextStyle, &text_property);
       } else {
-         sent_ev.xselectionrequest.property = None;
-      } */
-      XSendEvent(mainDisplay,requestor,False,NoEventMask,&sent_ev);
+         XmbTextListToTextProperty(mainDisplay, &(dsCutBuffer.s), 1,
+               XCompoundTextStyle, &text_property);
+      }
+      XChangeProperty(mainDisplay, requestor, property, target,
+            text_property.format, PropModeReplace, text_property.value,
+            text_property.nitems);
+      sent_ev.xselection.property = property;
    }
+   XSendEvent(mainDisplay, requestor, False, NoEventMask, &sent_ev);
 }
-#endif /* ICCCM_TEST */
 
 static XComposeStatus c_stat;
 
@@ -733,16 +816,17 @@ int mainWinEventHandler(input)
          return INVALID;
       }
       switch (key_sym) {
-      case XK_Left: PrevPage(); return INVALID;
-      case XK_KP_Left: PrevPage(); return INVALID;
-      case XK_Right: NextPage(); return INVALID;
-      case XK_KP_Right: NextPage(); return INVALID;
+      case XK_Left: PrevSlide(); return INVALID;
+      case XK_KP_Left: PrevSlide(); return INVALID;
+      case XK_Right: NextSlide(); return INVALID;
+      case XK_KP_Right: NextSlide(); return INVALID;
       default: break;
       }
    } else if (input->type == SelectionRequest) {
-#ifdef ICCCM_TEST
       HandleSelectionRequest(input);
-#endif /* ICCCM_TEST */
+      return INVALID;
+   } else if (input->type == SelectionClear) {
+      HandleSelectionClear(input);
       return INVALID;
    } else if (input->type == UnmapNotify) {
       Iconify();
@@ -811,6 +895,19 @@ int mainWinEventHandler(input)
                   key_sym = (KeySym)(key_ev.keycode);
                   TranslateKeys(buf, &key_sym);
                }
+               /*
+                * This is probabl not needed.
+                *
+                * ClearCopyUTF8Info(); 
+                * if (textHighlight) {
+                *    if (!CanCopyHighLightedTextAsUTF8Strings(NULL)) {
+                *       ClearCopyUTF8Info();
+                *    } else if (gstCopyUTF8Info.double_byte_valid &&
+                *          !gstCopyUTF8Info.single_byte_valid) {
+                *       CalcSingleByteInfoForCopyUTF8(NULL); 
+                *    }
+                * }
+                */
                HandleTextFromBuffer(&key_ev, buf, key_sym, has_ch);
             }
          } else {
@@ -870,11 +967,10 @@ void InitPaperSize()
    psPageHeightInInch[1] = ((float)onePageWidth)/((float)ONE_INCH);
 }
 
-static Window	savedMainWindow=None, savedDrawWindow=None;
 static Pixmap	wmIconPixmap=None, wmIconPixmapMask=None;
 
 static
-int GetIconPixmap(pPixmap, pPixmapMask)
+int GetWMIconPixmap(pPixmap, pPixmapMask)
    Pixmap *pPixmap, *pPixmapMask;
 {
    Pixmap pixmap=None, bitmap=None;
@@ -882,7 +978,8 @@ int GetIconPixmap(pPixmap, pPixmapMask)
    int rc=0, image_w=0, image_h=0, w=0, h=0, ncolors=0, chars_per_pixel=0;
    int first_pixel_is_bg=FALSE, *pixels=NULL;
    char fname[MAXPATHLENGTH], *color_char=NULL, **color_str=NULL;
-   char *c_ptr=XGetDefault(mainDisplay,TOOL_NAME, "IconPixmap"), *xpm_data=NULL;
+   char *xpm_data=NULL;
+   char *c_ptr=XGetDefault(mainDisplay,TOOL_NAME, "WMIconPixmap");
 
    *pPixmap = *pPixmapMask = None;
    if (c_ptr == NULL) return FALSE;
@@ -919,6 +1016,7 @@ int GetIconPixmap(pPixmap, pPixmapMask)
       for (i=0; i < ncolors; i++) free(color_str[i]);
       free(color_str);
    }
+   if (pixels != NULL) free(pixels);
    *pPixmap = pixmap;
    *pPixmapMask = bitmap;
    return TRUE;
@@ -1003,13 +1101,37 @@ void InitTmpDir()
    }
 }
 
+static
+int InitAntiAlias()
+   /*
+    * Get here because fullTrueColorMode is TRUE.  So, by default, we
+    *         will use antialiased fonts, unless Tgif.NoAntialias is set
+    *         to TRUE.
+    * Need to add code to talk to the xft server.
+    */
+{
+#ifdef NOT_DEFINED
+   char *c_ptr=NULL;
+
+   noAntiAlias = FALSE;
+   if ((c_ptr=XGetDefault(mainDisplay, TOOL_NAME, "NoAntialias")) !=
+         NULL && UtilStrICmp(c_ptr, "true") == 0) {
+      noAntiAlias = TRUE;
+   }
+#endif /* NOT_DEFINED */
+   /*
+    * Add code here to talk to the xft server.
+    */
+   return TRUE;
+}
+
 void Setup()
 {
    int bitmask=0, x_neg_in_def=FALSE, y_neg_in_def=FALSE;
    int x_in_cmdline=FALSE, y_in_cmdline=FALSE, cur_y=0, i, x_offset=0;
-   int bg_pixel=(-1);
+   int bg_pixel=(-1), def_x_neg=0, def_y_neg=0, paper_size_set=FALSE;
    char *c_ptr=NULL;
-   int def_x_neg=0, def_y_neg=0, paper_size_set;
+   double dval=(double)0;
    XWMHints wmhints;
    XSizeHints sizehints;
    XClassHint classhints;
@@ -1023,7 +1145,7 @@ void Setup()
    }
 #endif /* _TGIF_DBG */
    if ((c_ptr=XGetDefault(mainDisplay, TOOL_NAME, "Synchronize")) != NULL) {
-      if (UtilStrICmp(c_ptr, "on") == 0) {
+      if (UtilStrICmp(c_ptr, "on") == 0 || UtilStrICmp(c_ptr, "true") == 0) {
          XSynchronize(mainDisplay, True);
       }
    }
@@ -1087,6 +1209,8 @@ void Setup()
          whereToPrint = PPM_FILE;
       } else if (UtilStrICmp(c_ptr, "NetList") == 0) {
          whereToPrint = NETLIST_FILE;
+      } else if (UtilStrICmp(c_ptr, "SVG") == 0) {
+         whereToPrint = SVG_FILE;
       } else {
          fprintf(stderr, TgLoadString(STID_INVALID_XDEF_USE_ALT_STR),
                TOOL_NAME, "WhereToPrint", c_ptr, "EPS");
@@ -1165,10 +1289,11 @@ void Setup()
          splineTol = 9;
       }
    }
-   splineRubberband = TRUE;
    if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"SplineRubberband")) != NULL) {
       if (UtilStrICmp(c_ptr, "false") == 0) {
-         splineRubberband = FALSE;
+         sprintf(gszMsgBox, TgLoadString(STID_NAMED_XDEF_IS_OBSOLETE),
+               TOOL_NAME, "SplineRubberband");
+         MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
       }
    }
    saveTmpOnReturn = TRUE;
@@ -1228,7 +1353,7 @@ void Setup()
          btn1Warp = TRUE;
       } else if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,
             "QuickWarp")) != NULL) {
-         if (UtilStrICmp("on", c_ptr) == 0) {
+         if (UtilStrICmp("on", c_ptr) == 0 || UtilStrICmp("true", c_ptr) == 0) {
             btn1Warp = TRUE;
          }
       } else if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,
@@ -1341,19 +1466,19 @@ void Setup()
          }
       }
    }
-#ifdef _TGIF_WB2
-   if (cmdLineNoChatWindow) {
-      noChatWindow = TRUE;
-   } else {
-      noChatWindow = FALSE;
-      if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"NoChatWindow")) != NULL) {
-         if (UtilStrICmp("true", c_ptr) == 0) {
-            noChatWindow = TRUE;
+   if (gstWBInfo.do_whiteboard) {
+      if (cmdLineNoChatWindow) {
+         noChatWindow = TRUE;
+      } else {
+         noChatWindow = FALSE;
+         if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"NoChatWindow"))
+               != NULL) {
+            if (UtilStrICmp("true", c_ptr) == 0) {
+               noChatWindow = TRUE;
+            }
          }
       }
    }
-#endif /* _TGIF_WB2 */
-
    scrollBarW = baseScrollBarW = SCROLLBAR_W;
    if ((c_ptr=XGetDefault(mainDisplay, TOOL_NAME, "ScrollBarWidth")) != NULL) {
       baseScrollBarW = atoi(c_ptr);
@@ -1376,10 +1501,13 @@ void Setup()
    }
    /* do not translate -- program constants */
    wmDeleteWindowAtom = XInternAtom(mainDisplay, "WM_DELETE_WINDOW", False);
+   targetsAtom = XInternAtom(mainDisplay, "TARGETS", False);
    utf8StringAtom = XInternAtom(mainDisplay, "UTF8_STRING", False);
+   tgifProtocolAtom = XInternAtom(mainDisplay, TGIF_PROTOCOL_ATOM, False);
+   tmpSelectionAtom = XInternAtom(mainDisplay, "TGIF_TMP_SELECTION_ATOM",
+         False);
    compoundTextAtom = XInternAtom(mainDisplay, "COMPOUND_TEXT", False);
    textAtom = XInternAtom(mainDisplay, "TEXT", False);
-   stringAtom = XInternAtom(mainDisplay, "STRING", False);
    executeCmdByIDAtom = XInternAtom(mainDisplay, "TGIF_EXECUTE_CMD_BY_ID",
          False);
 
@@ -1593,7 +1721,17 @@ void Setup()
 
    colorDisplay = (cmdLineBW || DisplayPlanes(mainDisplay,mainScreen)==1 ||
          mainVisual->class==StaticGray) ? FALSE : TRUE;
-
+   fullTrueColorMode = FALSE;
+   if (colorDisplay && mainVisual->class == TrueColor) {
+      if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"EnableTrueColorImages")) !=
+            NULL && UtilStrICmp(c_ptr, "true") == 0) {
+         fullTrueColorMode = TRUE;
+      }
+   }
+   noAntiAlias = TRUE;
+   if (fullTrueColorMode) {
+      InitAntiAlias();
+   }
    threeDLook = colorDisplay;
    if (threeDLook && (c_ptr=XGetDefault(mainDisplay, TOOL_NAME, "3DLook")) !=
          NULL && UtilStrICmp(c_ptr, "false") == 0) {
@@ -1637,6 +1775,42 @@ void Setup()
          "Btn2PopupMainMenu")) != NULL && UtilStrICmp(c_ptr, "false") == 0) {
       btn2PopupMainMenu = FALSE;
    }
+   defRightMarginEnabled = INVALID;
+   if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"EnableRightMargin")) != NULL) {
+      if (UtilStrICmp("true", c_ptr) == 0) {
+         defRightMarginEnabled = TRUE;
+      } else {
+         defRightMarginEnabled = FALSE;
+      }
+   }
+   if (gridSystem == ENGLISH_GRID) {
+      defRightMargin = ONE_INCH;
+   } else {
+      defRightMargin = (ONE_CM<<1)+(ONE_CM>>1);
+   }
+   dval = ((double)defRightMargin) * ((double)100) * ((double)TIK_PER_PIXEL) /
+         ((double)printMag);
+   defRightMargin = round(dval);
+   if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"RightMargin")) != NULL) {
+      char spec_buf[40];
+
+      UtilStrCpyN(spec_buf, sizeof(spec_buf), c_ptr);
+      UtilTrimBlanks(spec_buf);
+      if (GetDimensionInDouble(spec_buf, TRUE, &dval)) {
+         dval = dval * ((double)100) * ((double)TIK_PER_PIXEL) /
+               ((double)printMag);
+         defRightMargin = round(dval);
+      } else {
+         fprintf(stderr, TgLoadString(STID_INVALID_XDEF_USE_ALT_STR),
+               TOOL_NAME, "RightMargin", c_ptr,
+               (gridSystem==ENGLISH_GRID) ? "1 in" : "2.5 cm");
+         fprintf(stderr, "\n");
+      }
+   }
+   rightMargin = defRightMargin;
+   rightMarginEnabled = defRightMarginEnabled;
+   rightMarginActive = (rightMarginEnabled == TRUE);
+
    InitFonts();
 
    initialMenubarWindowH = defaultFontHeight;
@@ -1746,7 +1920,7 @@ void Setup()
       }
       drawWinW = initDrawWinW = mainWinW-(brdrW<<1);
       drawWinH = initDrawWinH = mainWinH-(brdrW<<1);
-      if (msgWindowW <= 0) msgWindowW = 2;
+      if (msgWindowW <= 0) msgWindowW = 1;
    } else {
       sizehints.min_width =
             (noModeWindow ? 0 : modeWindowW+(brdrW<<1))+
@@ -1779,16 +1953,20 @@ void Setup()
    } else if (!y_in_cmdline && y_neg_in_def) {
       sizehints.y = def_y_neg+DisplayHeight(mainDisplay,mainScreen)-mainWinH-1;
    }
-#if defined(_TANGRAM_II) || defined(_TGIF_WB2)
-   if ( cmdLineParentWindowID > 0 ) {
-      rootWindow = cmdLineParentWindowID;
-   }
-#endif /* defined(_TANGRAM_II) || defined(_TGIF_WB2) */
    if ((mainWindow=XCreateSimpleWindow(mainDisplay, rootWindow,
          sizehints.x, sizehints.y, sizehints.width, sizehints.height,
          brdrW, myBorderPixel, myBgPixel)) == 0) {
       FailToCreateWindowMessage("Setup()", NULL, TRUE);
    }
+   if (gstWBInfo.do_whiteboard) {
+      if ( cmdLineParentWindowID > 0 ) {
+         XReparentWindow(mainDisplay, mainWindow, cmdLineParentWindowID, 0, 0);
+      }
+   } else {
+      if ( cmdLineTgrm2 && cmdLineParentWindowID > 0 ) {
+         XReparentWindow(mainDisplay, mainWindow, cmdLineParentWindowID, 0, 0);
+      }
+   }
    if (newColormapUsed) {
       XSetWindowColormap(mainDisplay, mainWindow, mainColormap);
    }
@@ -1930,31 +2108,21 @@ void Setup()
       }
    }
    InitTidget();
+   InitColorWheel();
+   InitInsertSymbol();
+
+   if (!CreateChatSubWindows()) {
+      FailToCreateWindowMessage("Setup()", NULL, TRUE);
+   }
    if (!noChatWindow) {
-      if (!CreateChatSubWindows()) {
-         FailToCreateWindowMessage("Setup()", NULL, TRUE);
-      }
       cur_y += chatWindowH + (brdrW<<1);
    }
    if (!noStatusWindow) {
       cur_y += statusWindowH + (brdrW<<1);
    }
    if (canvasWindowOnly) {
-      savedMainWindow = mainWindow;
-      savedDrawWindow = drawWindow;
-
-      if ((mainWindow=XCreateSimpleWindow(mainDisplay, rootWindow,
-            sizehints.x, sizehints.y, mainWinW, mainWinH,
-            brdrW, myBorderPixel, bg_pixel)) == 0) {
-         FailToCreateWindowMessage("Setup()", NULL, TRUE);
-      }
-      if (newColormapUsed) {
-         XSetWindowColormap(mainDisplay, mainWindow, mainColormap);
-      }
-      if ((drawWindow=XCreateSimpleWindow(mainDisplay, mainWindow,
-            0, 0, drawWinW, drawWinH, brdrW, myBorderPixel, myBgPixel)) == 0) {
-         FailToCreateWindowMessage("Setup()", NULL, TRUE);
-      }
+      XMoveResizeWindow(mainDisplay, drawWindow, 0, 0, drawWinW, drawWinH);
+      XResizeWindow(mainDisplay, mainWindow, mainWinW, mainWinH);
    }
 
 #ifdef NOTR4MODE
@@ -1993,10 +2161,9 @@ void Setup()
    InitNKF();
 #endif /* ~_NO_NKF */
 
-#ifdef _TANGRAM_II
-   InitTangram2();
-#endif /* _TANGRAM_II */
-
+   if (cmdLineTgrm2) {
+      InitTangram2();
+   }
    InitShortCut();
    InitHtml();
    InitRecentFiles();
@@ -2028,8 +2195,9 @@ void Setup()
             PropertyChangeMask);
    } else {
       XMapWindow(mainDisplay, titleWindow);
-      XSelectInput(mainDisplay, titleWindow,
-            ExposureMask | EnterWindowMask | ButtonPressMask);
+      XSelectInput(mainDisplay, titleWindow, ExposureMask |
+            EnterWindowMask | LeaveWindowMask | ButtonPressMask |
+            PointerMotionMask);
 
       if (menubarWindow != None) XMapWindow(mainDisplay, menubarWindow);
       XSelectInput(mainDisplay, menubarWindow, ExposureMask |
@@ -2093,12 +2261,12 @@ void Setup()
 
       if (!noChatWindow) {
          XMapWindow(mainDisplay, chatWindow);
-         XSelectInput(mainDisplay, chatWindow,
-               KeyPressMask | ButtonPressMask | ExposureMask |
-               StructureNotifyMask | EnterWindowMask | LeaveWindowMask);
-
-         MapChatSubWindows();
       }
+      MapChatSubWindows();
+      XSelectInput(mainDisplay, chatWindow,
+            KeyPressMask | ButtonPressMask | ExposureMask |
+            StructureNotifyMask | EnterWindowMask | LeaveWindowMask);
+
       if (!noStatusWindow) {
          XMapWindow(mainDisplay, userRedrawWindow);
          XSelectInput(mainDisplay, userRedrawWindow, ButtonPressMask |
@@ -2127,8 +2295,9 @@ void Setup()
             PropertyChangeMask);
       XMapWindow(mainDisplay, drawWindow); 
    } else {
-      XSelectInput(mainDisplay, titleWindow,
-            ExposureMask | EnterWindowMask | ButtonPressMask);
+      XSelectInput(mainDisplay, titleWindow, ExposureMask |
+            EnterWindowMask | LeaveWindowMask | ButtonPressMask |
+            PointerMotionMask);
       XMapWindow(mainDisplay, titleWindow);
 
       XSelectInput(mainDisplay, menubarWindow, ExposureMask |
@@ -2193,13 +2362,14 @@ void Setup()
       XSelectInput(mainDisplay, dummyWindow2, ExposureMask | EnterWindowMask);
       XMapWindow(mainDisplay, dummyWindow2);
 
+      XSelectInput(mainDisplay, chatWindow,
+            KeyPressMask | ButtonPressMask | ExposureMask |
+            StructureNotifyMask | EnterWindowMask | LeaveWindowMask);
       if (!noChatWindow) {
          XMapWindow(mainDisplay, chatWindow);
-         MapChatSubWindows();
-         XSelectInput(mainDisplay, chatWindow,
-               KeyPressMask | ButtonPressMask | ExposureMask |
-               StructureNotifyMask | EnterWindowMask | LeaveWindowMask);
       }
+      MapChatSubWindows();
+
       if (!noStatusWindow) {
          XSelectInput(mainDisplay, userRedrawWindow, ButtonPressMask |
                ExposureMask | EnterWindowMask);
@@ -2268,7 +2438,7 @@ void Setup()
             NULL) || UtilStrICmp(c_ptr, "false") != 0) {
          wmhints.icon_mask = None;
          wmhints.initial_state = NormalState;
-         if (!GetIconPixmap(&wmIconPixmap, &wmIconPixmapMask)) {
+         if (!GetWMIconPixmap(&wmIconPixmap, &wmIconPixmapMask)) {
             wmhints.icon_pixmap = wmIconPixmap =
                   XCreateBitmapFromData(mainDisplay, mainWindow,
                   (char*)tgificon_bits, tgificon_width, tgificon_height);
@@ -2310,13 +2480,12 @@ void CleanUpResiduals()
       XFreePixmap(mainDisplay, wmIconPixmapMask);
       wmIconPixmapMask = None;
    }
-   if (savedMainWindow != None) XDestroyWindow(mainDisplay, savedMainWindow);
    CleanUpPaperSize();
    if (displayName != NULL) {
       UtilFree(displayName);
       displayName = NULL;
    }
-   if (btn1_bits) { }
+   if (btn1_bits == NULL) { }
 }
 
 int TieLooseEnds()
