@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/setup.c,v 1.58 2009/09/20 23:42:49 cvsps Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/setup.c,v 1.71 2009/11/08 00:25:03 cvsps Exp $
  */
 
 #define _INCLUDE_FROM_SETUP_C_
@@ -91,6 +91,7 @@
 #include "tdgtlist.e"
 #include "text.e"
 #include "tgcwheel.e"
+#include "tginssym.e"
 #include "tidget.e"
 #include "util.e"
 #include "version.e"
@@ -524,6 +525,7 @@ void DoReconfigure()
    if (pageWindowW > 0) {
       XMoveResizeWindow(mainDisplay, pageWindow, 0, cur_y, pageWindowW,
             scrollBarW+(brdrW<<1));
+      MakeCurPageTabVisible();
    } else {
       XMoveWindow(mainDisplay, pageWindow, 0, cur_y);
    }
@@ -790,8 +792,13 @@ void HandleSelectionRequest(input)
    } else if (target == compoundTextAtom) {
       XTextProperty text_property;
 
-      XmbTextListToTextProperty(mainDisplay, &(dsCutBuffer.s), 1,
-            XCompoundTextStyle, &text_property);
+      if (cutBufferIsUTF8) {
+         Xutf8TextListToTextProperty(mainDisplay, &(dsCutBuffer.s), 1,
+               XCompoundTextStyle, &text_property);
+      } else {
+         XmbTextListToTextProperty(mainDisplay, &(dsCutBuffer.s), 1,
+               XCompoundTextStyle, &text_property);
+      }
       XChangeProperty(mainDisplay, requestor, property, target,
             text_property.format, PropModeReplace, text_property.value,
             text_property.nitems);
@@ -1022,6 +1029,7 @@ int GetWMIconPixmap(pPixmap, pPixmapMask)
       for (i=0; i < ncolors; i++) free(color_str[i]);
       free(color_str);
    }
+   if (pixels != NULL) free(pixels);
    *pPixmap = pixmap;
    *pPixmapMask = bitmap;
    return TRUE;
@@ -1134,9 +1142,9 @@ void Setup()
 {
    int bitmask=0, x_neg_in_def=FALSE, y_neg_in_def=FALSE;
    int x_in_cmdline=FALSE, y_in_cmdline=FALSE, cur_y=0, i, x_offset=0;
-   int bg_pixel=(-1);
+   int bg_pixel=(-1), def_x_neg=0, def_y_neg=0, paper_size_set=FALSE;
    char *c_ptr=NULL;
-   int def_x_neg=0, def_y_neg=0, paper_size_set;
+   double dval=(double)0;
    XWMHints wmhints;
    XSizeHints sizehints;
    XClassHint classhints;
@@ -1780,6 +1788,42 @@ void Setup()
          "Btn2PopupMainMenu")) != NULL && UtilStrICmp(c_ptr, "false") == 0) {
       btn2PopupMainMenu = FALSE;
    }
+   defRightMarginEnabled = INVALID;
+   if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"EnableRightMargin")) != NULL) {
+      if (UtilStrICmp("true", c_ptr) == 0) {
+         defRightMarginEnabled = TRUE;
+      } else {
+         defRightMarginEnabled = FALSE;
+      }
+   }
+   if (gridSystem == ENGLISH_GRID) {
+      defRightMargin = ONE_INCH;
+   } else {
+      defRightMargin = (ONE_CM<<1)+(ONE_CM>>1);
+   }
+   dval = ((double)defRightMargin) * ((double)100) * ((double)TIK_PER_PIXEL) /
+         ((double)printMag);
+   defRightMargin = round(dval);
+   if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"RightMargin")) != NULL) {
+      char spec_buf[40];
+
+      UtilStrCpyN(spec_buf, sizeof(spec_buf), c_ptr);
+      UtilTrimBlanks(spec_buf);
+      if (GetDimensionInDouble(spec_buf, TRUE, &dval)) {
+         dval = dval * ((double)100) * ((double)TIK_PER_PIXEL) /
+               ((double)printMag);
+         defRightMargin = round(dval);
+      } else {
+         fprintf(stderr, TgLoadString(STID_INVALID_XDEF_USE_ALT_STR),
+               TOOL_NAME, "RightMargin", c_ptr,
+               (gridSystem==ENGLISH_GRID) ? "1 in" : "2.5 cm");
+         fprintf(stderr, "\n");
+      }
+   }
+   rightMargin = defRightMargin;
+   rightMarginEnabled = defRightMarginEnabled;
+   rightMarginActive = (rightMarginEnabled == TRUE);
+
    InitFonts();
 
    initialMenubarWindowH = defaultFontHeight;
@@ -2078,6 +2122,7 @@ void Setup()
    }
    InitTidget();
    InitColorWheel();
+   InitInsertSymbol();
 
    if (!CreateChatSubWindows()) {
       FailToCreateWindowMessage("Setup()", NULL, TRUE);
