@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/spline.c,v 1.42 2009/09/15 21:47:46 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/spline.c,v 1.44 2009/10/08 17:25:21 william Exp $
  */
 
 #define _INCLUDE_FROM_SPLINE_C_
@@ -952,8 +952,8 @@ XPoint *MakeSplinePolygonVertex(Level, Curved, N, XOff, YOff, NumVs, Vs)
 }
 
 static
-XPoint *MakeDoubleSplinePolygonVertex(Curved, N, XOff, YOff, NumVs, Vs)
-   int Curved, *N, XOff, YOff, NumVs;
+XPoint *DoMakeDoubleIntSplinePolygonVertex(N, XOff, YOff, NumVs, Vs)
+   int *N, XOff, YOff, NumVs;
    DoublePoint *Vs;
 {
    double mx1, my1, mx2, my2, mx3, my3, mx4, my4, x1, y1, x2, y2, x, y;
@@ -1633,7 +1633,9 @@ XPoint *MakeDoubleIntSplinePolygonVertex(N, CntrlN, CntrlVs, XOff, YOff,
    int *N, *CntrlN, XOff, YOff, NumVs;
    DoublePoint **CntrlVs, *Vs;
 {
-   int x_off, y_off;
+   int x_off=0, y_off=0;
+   int saved_tighter_splines=tighterStructSplines;
+   XPoint *pxp=NULL;
 
    x_off = (zoomedIn ? XOff : (XOff>>zoomScale)<<zoomScale);
    y_off = (zoomedIn ? YOff : (YOff>>zoomScale)<<zoomScale);
@@ -1667,8 +1669,13 @@ XPoint *MakeDoubleIntSplinePolygonVertex(N, CntrlN, CntrlVs, XOff, YOff,
    Gaussian(NumVs);
    *CntrlVs = DoubleClosedControlPts(NumVs, CntrlN);
 
-   return MakeDoubleSplinePolygonVertex(LT_INTSPLINE, N, XOff, YOff, *CntrlN,
-         *CntrlVs);
+   if (saved_tighter_splines) {
+      tighterStructSplines = FALSE;
+   }
+   pxp = DoMakeDoubleIntSplinePolygonVertex(N, XOff, YOff, *CntrlN, *CntrlVs);
+   tighterStructSplines = saved_tighter_splines;
+
+   return pxp;
 }
 
 void DumpCurvedPolyPoints(FP, Curved, NumPts, V, Indent)
