@@ -2,7 +2,7 @@
  * Author:      Daniel Ratton Figueiredo <ratton@land.ufrj.br>
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2006, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2009, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -29,7 +29,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/tangram2.c,v 1.8 2006/05/30 02:59:32 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/tangram2.c,v 1.17 2009/10/04 03:10:44 william Exp $
  */
 
 #define _INCLUDE_FROM_TANGRAM2_C_
@@ -42,6 +42,7 @@
 #include "auxtext.e"
 #include "box.e"
 #include "cmd.e"
+#include "color.e"
 #include "dialog.e"
 #include "drawing.e"
 #include "exec.e"
@@ -61,8 +62,6 @@
 
 char	cmdLineGenParser[MAXSTRING];
 
-#ifdef _TANGRAM_II
-
 #define DO_CMD(cmd) ExecACommandFromBuffer(cmd,NULL)
 
 static TgMenuItemInfo tangram2MenuItemInfo[] = {
@@ -425,7 +424,7 @@ void ExecStartSimulator(obj_ptr, orig_cmd)
     char *service = "6743";
 
     strcpy( host, "localhost" );
-    bzero( (char *)&sin, sizeof( sin ) );
+    memset( (char *)&sin, 0, sizeof( sin ) );
     sin.sin_family = AF_INET;
     if( (pse = getservbyname( service, protocol )) != NULL )
         sin.sin_port = pse->s_port;
@@ -501,7 +500,7 @@ void ExecStartSimulator(obj_ptr, orig_cmd)
     {
         sin_size = sizeof( sin );
         if( recvfrom( s, &buffer, sizeof(buffer), 0, (struct sockaddr *)&sin,
-                      (socklen_t *)(&sin_size) ) < 0 )
+                      (void*)(&sin_size) ) < 0 )
         {
             perror( "TGIF: recv" );
             return;
@@ -1303,7 +1302,7 @@ static char *GenerateChainExec[] = {
 };
 
 static char *GenerateChainWriteModel[] = {
-  "find_obj_names(obj_list,\"\",\"Declaration=*\");",
+  "find_obj_names_on_all_pages(obj_list,\"Declaration=*\");",
   "get_line_in_attr(total_objs,obj_list,0);",
   "for_i(i,1,$(total_objs),1,write_obj);",
   NULL
@@ -1607,6 +1606,10 @@ char *mode;
    tmp_box_obj = topObj;
 
    tmp_box_obj->color = 9;
+   if (mainDisplay != NULL) {
+      UtilStrCpyN(tmp_box_obj->color_str, sizeof(tmp_box_obj->color_str),
+            colorMenuItems[tmp_box_obj->color]);
+   }
    tmp_box_obj->detail.rcb->width = 2;
    y += 5;
 
@@ -1759,7 +1762,7 @@ int max_arg_length;
     int i, j;
     char *ptr;
 
-    bzero( dst, no_args * max_arg_length );    
+    memset( dst, 0, no_args * max_arg_length );    
     ptr = args;
     /* Remove leading " and spaces */
     while( *ptr == '\"' || *ptr == ' ' )
@@ -2050,4 +2053,3 @@ void Tangram2UpdateInitAttr(port_obj, signal_name)
    UpdTextBBox(attr_ptr->obj);
 }
 
-#endif /* _TANGRAM_II */
