@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2006, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2009, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/tdgtsedt.c,v 1.3 2006/02/10 23:21:21 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/tdgtsedt.c,v 1.10 2009/06/10 18:52:44 cvsps Exp $
  */
 
 #define _INCLUDE_FROM_TDGTSEDT_C_
@@ -49,6 +49,7 @@
 #include "raster.e"
 #include "rect.e"
 #include "setup.e"
+#include "tdgtbase.e"
 #include "tdgtsedt.e"
 #include "text.e"
 #include "tidget.e"
@@ -179,13 +180,15 @@ void TdgtSmplEditDrawCaption(pTdgtSmplEdit)
    (pf_draw_string_func)(mainDisplay, pTdgtSmplEdit->pti->tci.win,
          gTidgetManager.gc, x, y, psz_start, strlen(psz_start));
 
-   x = (pf_text_width_func)(font_ptr, psz_start, strlen(psz_start)) +
-         pTdgtSmplEdit->client_area.x;
-   XSetForeground(mainDisplay, gTidgetManager.gc, myFgPixel);
-   XDrawLine(mainDisplay, pTdgtSmplEdit->pti->tci.win, gTidgetManager.gc, x+1,
-         pTdgtSmplEdit->client_area.y, x+1,
-         pTdgtSmplEdit->client_area.y+pTdgtSmplEdit->client_area.h);
-   XSetFont(mainDisplay, gTidgetManager.gc, defaultFontPtr->fid);
+   if (TidgetIsFocus(pTdgtSmplEdit->pti)) {
+      x = (pf_text_width_func)(font_ptr, psz_start, strlen(psz_start)) +
+            pTdgtSmplEdit->client_area.x;
+      XSetForeground(mainDisplay, gTidgetManager.gc, myFgPixel);
+      XDrawLine(mainDisplay, pTdgtSmplEdit->pti->tci.win, gTidgetManager.gc,
+            x+1, pTdgtSmplEdit->client_area.y+1, x+1,
+            pTdgtSmplEdit->client_area.y+pTdgtSmplEdit->client_area.h-1);
+      TidgetDrawFocusRect(pTdgtSmplEdit->pti, &pTdgtSmplEdit->client_area);
+   }
    TidgetManagerResetGC();
 }
 
@@ -249,11 +252,12 @@ int TdgtSmplEditEventHandler(pti, input, handling_pti)
       if (gnInputMethod != TGIM_NONE &&
             /*
              * Cannot call: InputMethodTypeMatched(canvasFontDoubleByte) &&
-             *                           */
+             */
             tgIMExpectLookupString(mainDisplay, pTdgtSmplEdit->pti->tci.win)) {
          if (tgIMHandleLookupString(mainDisplay, pTdgtSmplEdit->pti->tci.win,
                key_ev, buf, sizeof(buf), &key_sym, &has_ch)) {
             if (has_ch > 0) buf[has_ch] = '\0';
+            has_ch = HandleDoubleByteUTF8Chars(buf, has_ch);
          }
          TranslateKeys(buf, &key_sym);
       } else {
@@ -286,6 +290,19 @@ int TdgtSmplEditEventHandler(pti, input, handling_pti)
          TdgtSmplEditAppendStr(pTdgtSmplEdit, buf, 1);
          return TRUE;
       }
+   } else if (input->type == ButtonPress) {
+      if (pTdgtSmplEdit->cursor_shown) {
+         /* already has focus, don't do anythingn */
+      } else {
+         TdgtBase *pTdgtBase=
+               (TdgtBase*)(pTdgtSmplEdit->pti->tci.parent_tidgetinfo->tidget);
+
+         if (TidgetSendCmd(pTdgtBase->pti, TDGTCMD_CAN_HANDLE_FOCUS_CHANGE,
+               0, NULL)) {
+            TidgetSendCmd(pTdgtBase->pti, TDGTCMD_REMOVE_FOCUS, 0, NULL);
+         }
+         TdgtSmplEditSetFocus(pTdgtSmplEdit, TRUE);
+      }
    }
    return FALSE;
 }
@@ -425,6 +442,9 @@ TdgtSmplEdit *CreateTdgtSmplEdit(parent_win, parent_tidgetinfo, ctl_id, x, y,
       FailToCreateWindowMessage("CreateTdgtSmplEdit()", NULL, TRUE);
       return NULL;
    }
+   XSelectInput(mainDisplay, pTdgtSmplEdit->pti->tci.win,
+         ButtonReleaseMask | ButtonPressMask | KeyPressMask | ExposureMask |
+         EnterWindowMask | LeaveWindowMask);
    SetTidgetInfoBasic(pTdgtSmplEdit->pti, TIDGET_TYPE_SEDT, pTdgtSmplEdit,
          parent_win, x, y, w, h, h_pad, v_pad, TGBS_LOWRED, caption);
    TidgetSetCallbacks(pTdgtSmplEdit->pti,
@@ -588,27 +608,31 @@ int TdgtSmplEditDelLastChar(pTdgtSmplEdit)
 /* --------------------- TdgtSmplEditSetFocus() --------------------- */
 
 static
-void TdgtSmplEditShowCursor(pTdgtSmplEdit, show)
+int TdgtSmplEditShowCursor(pTdgtSmplEdit, show)
    TdgtSmplEdit *pTdgtSmplEdit;
    int show;
+   /* return TRUE if changed */
 {
    if (pTdgtSmplEdit->cursor_shown != show) {
       pTdgtSmplEdit->cursor_shown = show;
-      RedrawTdgtSmplEdit(pTdgtSmplEdit->pti);
+      return TRUE;
    }
+   return FALSE;
 }
 
 void TdgtSmplEditSetFocus(pTdgtSmplEdit, nSet)
    TdgtSmplEdit *pTdgtSmplEdit;
    int nSet;
 {
-   TdgtSmplEditShowCursor(pTdgtSmplEdit, nSet);
-   if (nSet) {
-      TidgetSetHasFocus(TRUE);
-      TidgetSetFocusWindow(pTdgtSmplEdit->pti->tci.win);
-   } else {
-      TidgetSetHasFocus(FALSE);
-      TidgetSetFocusWindow(None);
+   if (TdgtSmplEditShowCursor(pTdgtSmplEdit, nSet)) {
+      if (nSet) {
+         TidgetSetHasFocus(TRUE);
+         TidgetSetFocusWindow(pTdgtSmplEdit->pti->tci.win);
+      } else {
+         TidgetSetHasFocus(FALSE);
+         TidgetSetFocusWindow(None);
+      }
+      RedrawTdgtSmplEdit(pTdgtSmplEdit->pti);
    }
 }
 
