@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 2001-2003, William Chia-Wei Cheng.
+ * Copyright (C) 2001-2009, William Chia-Wei Cheng.
  *
  * This file may be distributed under the terms of the Q Public License
  * as defined by Trolltech AS of Norway and appearing in the file
@@ -15,7 +15,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/tgif.c,v 1.5 2005/01/15 00:23:24 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/tgif.c,v 1.16 2009/03/20 16:27:10 cvsps Exp $
  */
 
 #define _INCLUDE_FROM_TGIF_C_
@@ -33,6 +33,7 @@
 #include "names.e"
 #include "obj.e"
 #include "page.e"
+#include "raster.e"
 #include "setup.e"
 #include "strtbl.e"
 #include "tangram2.e"
@@ -125,9 +126,10 @@ void SetProgramName(FileName)
    char *c_ptr=UtilStrRChr(FileName, DIR_SEP);
 
    if (c_ptr != NULL && *(++c_ptr) != '\0') {
-      strcpy(progName, c_ptr);
+      UtilStrCpyN(progName, sizeof(progName), c_ptr);
    } else {
-      strcpy(progName, TOOL_NAME);
+      UtilStrCpyN(progName, sizeof(progName), TOOL_NAME);
+      UtilStrLower(progName);
    }
 }
 
@@ -261,6 +263,15 @@ int PrTgifLoad(FileName)
       fprintf(stderr, "\n");
       return FALSE;
    }
+   if (loadedCurPageNum > 0 && curPage != NULL && lastPageNum > 1) {
+      /*
+       * Well, it should go to loadedCurPageNum because it was the page
+       * at which the file was saved.  But we go to lastPageNum to stay
+       * compatible with Tangram-II since it was expected for tgif to
+       * behave this way.
+       */
+      GotoPageNum(lastPageNum);
+   }
    return TRUE;
 }
 
@@ -272,6 +283,7 @@ void PrTgifDump(inbuf, tile_page_err_msg_id)
    if (inbuf != NULL && strcmp(inbuf, "-") != 0) {
       UtilStrCpyN(cmdLineOpenFile, sizeof(cmdLineOpenFile), inbuf);
    }
+   if (cmdLineNoShowPageInEPS) showPageInEPS = FALSE;
    if (*cmdLineFileToExec != '\0') {
       if (!cmdLineQuiet) {
          fprintf(stderr, TgLoadString(STID_EXECUTING_GIVEN_SCRIPT),
@@ -334,11 +346,15 @@ void DoPrTgif(argc, argv, from_prtgif)
    }
    InitPaperSize();
    InitTmpDir();
-   if (PRTGIF && !cmdLineOpenDisplay) InitDontReencode(NULL);
+   InitPatFill();
+   if (PRTGIF && !cmdLineOpenDisplay) {
+      InitDontReencode(NULL);
+      InitPSFontCharSubs();
+   }
    if (*cmdLineGenParser != '\0') {
-#ifdef _TANGRAM_II
-      InitNames();
-#endif /* _TANGRAM_II */
+      if (cmdLineTgrm2) {
+         InitNames();
+      }
    }
    if (argc <= 0) {
       if (cmdLineStdOut) {
@@ -348,6 +364,7 @@ void DoPrTgif(argc, argv, from_prtgif)
       } else if (strcmp(cmdLineFileToExec, "-") == 0) {
          if (cmdLineOpenDisplay) {
             JustInit(NULL, NULL);
+            Init2PatFill();
          }
          PrTgifDump(NULL, STID_CANT_ONEFPPAGE_IN_TILED_PAGE);
       } else {
@@ -360,14 +377,14 @@ void DoPrTgif(argc, argv, from_prtgif)
             if (len > 0) {
                if (inbuf[--len] == '\n') inbuf[len] = '\0';
                if (*cmdLineGenParser != '\0') {
-#ifdef _TANGRAM_II
-                  if (PrTgifLoad(inbuf)) {
-                     Tangram2GenerateChain(cmdLineGenParser);
-                     DelAllObj();
-                  } else {
-                     fprintf(stderr, "\n");
+                  if (cmdLineTgrm2) {
+                     if (PrTgifLoad(inbuf)) {
+                        Tangram2GenerateChain(cmdLineGenParser);
+                        DelAllObj();
+                     } else {
+                        fprintf(stderr, "\n");
+                     }
                   }
-#endif /* _TANGRAM_II */
                } else if (cmdLineDumpURL) {
                   UrlDump(inbuf);
                } else if (cmdLineDosEpsFilter) {
@@ -401,14 +418,14 @@ void DoPrTgif(argc, argv, from_prtgif)
       for ( ; argc > 0; argc--, argv++) {
          lastFile = (argc == 1);
          if (*cmdLineGenParser != '\0') {
-#ifdef _TANGRAM_II
-            if (PrTgifLoad(*argv)) {
-               Tangram2GenerateChain(cmdLineGenParser);
-               DelAllObj();
-            } else {
-               fprintf(stderr, "\n");
+            if (cmdLineTgrm2) {
+               if (PrTgifLoad(*argv)) {
+                  Tangram2GenerateChain(cmdLineGenParser);
+                  DelAllObj();
+               } else {
+                  fprintf(stderr, "\n");
+               }
             }
-#endif /* _TANGRAM_II */
          } else if (cmdLineDumpURL) {
             UrlDump(*argv);
             if (argc > 1) {
@@ -439,6 +456,7 @@ void DoPrTgif(argc, argv, from_prtgif)
    if (PRTGIF && !cmdLineOpenDisplay) CleanUpFonts();
    CleanUpPage();
    CleanUpResiduals();
+   CleanUpPatFill();
    CleanUpStrTable();
 }
 
