@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2006, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2009, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/tgif.c,v 1.8 2006/06/09 23:00:00 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/tgif.c,v 1.16 2009/03/20 16:27:10 cvsps Exp $
  */
 
 #define _INCLUDE_FROM_TGIF_C_
@@ -46,6 +46,7 @@
 #include "names.e"
 #include "obj.e"
 #include "page.e"
+#include "raster.e"
 #include "setup.e"
 #include "strtbl.e"
 #include "tangram2.e"
@@ -138,9 +139,10 @@ void SetProgramName(FileName)
    char *c_ptr=UtilStrRChr(FileName, DIR_SEP);
 
    if (c_ptr != NULL && *(++c_ptr) != '\0') {
-      strcpy(progName, c_ptr);
+      UtilStrCpyN(progName, sizeof(progName), c_ptr);
    } else {
-      strcpy(progName, TOOL_NAME);
+      UtilStrCpyN(progName, sizeof(progName), TOOL_NAME);
+      UtilStrLower(progName);
    }
 }
 
@@ -274,6 +276,15 @@ int PrTgifLoad(FileName)
       fprintf(stderr, "\n");
       return FALSE;
    }
+   if (loadedCurPageNum > 0 && curPage != NULL && lastPageNum > 1) {
+      /*
+       * Well, it should go to loadedCurPageNum because it was the page
+       * at which the file was saved.  But we go to lastPageNum to stay
+       * compatible with Tangram-II since it was expected for tgif to
+       * behave this way.
+       */
+      GotoPageNum(lastPageNum);
+   }
    return TRUE;
 }
 
@@ -348,14 +359,15 @@ void DoPrTgif(argc, argv, from_prtgif)
    }
    InitPaperSize();
    InitTmpDir();
+   InitPatFill();
    if (PRTGIF && !cmdLineOpenDisplay) {
       InitDontReencode(NULL);
       InitPSFontCharSubs();
    }
    if (*cmdLineGenParser != '\0') {
-#ifdef _TANGRAM_II
-      InitNames();
-#endif /* _TANGRAM_II */
+      if (cmdLineTgrm2) {
+         InitNames();
+      }
    }
    if (argc <= 0) {
       if (cmdLineStdOut) {
@@ -365,6 +377,7 @@ void DoPrTgif(argc, argv, from_prtgif)
       } else if (strcmp(cmdLineFileToExec, "-") == 0) {
          if (cmdLineOpenDisplay) {
             JustInit(NULL, NULL);
+            Init2PatFill();
          }
          PrTgifDump(NULL, STID_CANT_ONEFPPAGE_IN_TILED_PAGE);
       } else {
@@ -377,14 +390,14 @@ void DoPrTgif(argc, argv, from_prtgif)
             if (len > 0) {
                if (inbuf[--len] == '\n') inbuf[len] = '\0';
                if (*cmdLineGenParser != '\0') {
-#ifdef _TANGRAM_II
-                  if (PrTgifLoad(inbuf)) {
-                     Tangram2GenerateChain(cmdLineGenParser);
-                     DelAllObj();
-                  } else {
-                     fprintf(stderr, "\n");
+                  if (cmdLineTgrm2) {
+                     if (PrTgifLoad(inbuf)) {
+                        Tangram2GenerateChain(cmdLineGenParser);
+                        DelAllObj();
+                     } else {
+                        fprintf(stderr, "\n");
+                     }
                   }
-#endif /* _TANGRAM_II */
                } else if (cmdLineDumpURL) {
                   UrlDump(inbuf);
                } else if (cmdLineDosEpsFilter) {
@@ -418,14 +431,14 @@ void DoPrTgif(argc, argv, from_prtgif)
       for ( ; argc > 0; argc--, argv++) {
          lastFile = (argc == 1);
          if (*cmdLineGenParser != '\0') {
-#ifdef _TANGRAM_II
-            if (PrTgifLoad(*argv)) {
-               Tangram2GenerateChain(cmdLineGenParser);
-               DelAllObj();
-            } else {
-               fprintf(stderr, "\n");
+            if (cmdLineTgrm2) {
+               if (PrTgifLoad(*argv)) {
+                  Tangram2GenerateChain(cmdLineGenParser);
+                  DelAllObj();
+               } else {
+                  fprintf(stderr, "\n");
+               }
             }
-#endif /* _TANGRAM_II */
          } else if (cmdLineDumpURL) {
             UrlDump(*argv);
             if (argc > 1) {
@@ -456,6 +469,7 @@ void DoPrTgif(argc, argv, from_prtgif)
    if (PRTGIF && !cmdLineOpenDisplay) CleanUpFonts();
    CleanUpPage();
    CleanUpResiduals();
+   CleanUpPatFill();
    CleanUpStrTable();
 }
 
