@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2006, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2009, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/tgifdefs.h,v 1.3 2006/02/10 23:21:22 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/tgifdefs.h,v 1.9 2009/10/03 04:45:52 william Exp $
  */
 
 #ifndef _TGIFDEFS_H_
@@ -133,9 +133,16 @@
 #endif /* !defined(__ELF__) && !defined(mips) */
 #endif /* __NetBSD__ */
 
+#ifdef NOT_DEFINED
+/*
+ * On some Mac OS X systems, libdl.dylib exists but there is no <dlcfn.h>.
+ *     Not sure if someone deleted <dlfcn.h> or it's a problem with the
+ *     system.
+ */
 #ifdef HAVE_LIBDL
 #undef _NO_DL_SUPPORT
 #endif /* HAVE_LIBDL */
+#endif /* NOT_DEFINED */
 
 #ifndef _NO_DL_SUPPORT
 #include <dlfcn.h>
@@ -154,6 +161,10 @@
 #include <errno.h>
 #include <time.h>
 
+#ifdef HAVE_STDINT_H
+#include <stdint.h>
+#endif /* HAVE_STDINT_H */
+
 #ifdef _ENABLE_NLS
 #ifndef ENABLE_NLS
 #define ENABLE_NLS
