@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2006, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2009, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/types.h,v 1.7 2006/06/11 20:36:58 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/types.h,v 1.42 2009/09/30 14:53:55 william Exp $
  */
 
 #ifndef _TGIF_TYPES_H_
@@ -57,6 +57,14 @@ typedef struct {
    int	x, y;
 } IntPoint;
 
+typedef struct tagIntPointTriplet {
+   int earlier_valid, later_valid;
+   IntPoint hinge_pt;
+   IntPoint earlier_smooth_pt;
+   IntPoint later_smooth_pt; /* this point moves at first */
+   double ratio;  /* segment len earlier = (segment len later * ratio) */
+} IntPointTriplet;
+
 typedef struct {
    double	x, y;
 } DoublePoint;
@@ -160,6 +168,9 @@ typedef struct PolyRec {
    int			n;	/* number of points in the polyline */
    IntPoint		* vlist; /* absolute */
    char			* smooth;
+   int			ssn;	/* number of points in a structured spline */
+   IntPoint		* ssvlist; /* absolute */
+   char			* ssmooth;
    int			sn;	/* number of points in the spline polyline */
    XPoint		* svlist; /* offsets */
    int			asn;	/* spline polyline with arrows */
@@ -187,6 +198,9 @@ typedef struct PolygonRec {
    int			n;	/* number of points in the polygon */
    IntPoint		* vlist; /* absolute */
    char			* smooth;
+   int			ssn;	/* number of points in a structured spline */
+   IntPoint		* ssvlist; /* absolute */
+   char			* ssmooth;
    int			sn;	/* number of points in the spline polygon */
    XPoint		* svlist; /* offsets */
    int			intn;	/* interpolated spline original spec */
@@ -239,7 +253,7 @@ typedef struct StrSegRec {
    int	font, style, sz_unit;
    int	real_sz_unit; /* only used if Tgif.EditTextSize is used */
    int	double_byte, double_byte_mod_bytes, double_byte_vertical, direction;
-   int	underline_on;
+   int	underline_on, overline_on;
    int	w, asc, des, min_lbearing, max_rextra;
    int	read_only, orig_w, orig_h, orig_asc, orig_des; /* only if read_only */
    char	* font_name; /* only used in PRTGIF */
@@ -330,7 +344,7 @@ typedef struct TextRec {
    int			cached_zoom, cached_zoomed;
    struct XfrmMtrxRec	cached_ctm;
    struct BBRec		orig_bbox;
-   int			underline_y_offset;
+   int			underline_y_offset, overline_y_offset;
    /* these are new */
    /*
     * baseline_y is the real thing!  y of struct ObjRec is derived from it
@@ -341,15 +355,44 @@ typedef struct TextRec {
    struct MiniLinesRec	minilines;
 } * TextRecPtr;
 
+typedef struct tagStretchStructuredSplineInfo {
+   int hinge; /* TRUE if a hinge point is selected, FALSE otherwise */
+   int orig_hinge_index; /* this index into vlist */
+   int earlier_smooth_selected;
+   int prev_valid; /* FALSE if the first point of a poly is selected */
+   int next_valid; /* FALSE if the last point of a poly is selected */
+   IntPointTriplet ipt_prev, ipt, ipt_next;
+   /* for drawing */
+   int num_vs, num_vs2;
+   IntPoint vs[5], vs2[5];
+   char smooth[5], smooth2[5];
+   XPoint *sv, *sv2, *saved_sv, *saved_sv2;
+   int sn, sn2, saved_sn, saved_sn2;
+   /* for moving a vertex -- only one vertex can be selected */
+   int orig_abs_x, orig_abs_y, new_abs_x, new_abs_y, abs_dx, abs_dy;
+   int rotated_orig_abs_x, rotated_orig_abs_y;
+   int rotated_new_abs_x, rotated_new_abs_y;
+   int rotated_abs_dx, rotated_abs_dy;
+} StretchStructuredSplineInfo;
+
 typedef struct SelRec {
    struct ObjRec	* obj;
    struct SelRec	* next, * prev;
 } * SelRecPtr;
 
+typedef struct tagMoveVertexInfo {
+   int orig_abs_x, orig_abs_y, new_abs_x, new_abs_y, abs_dx, abs_dy;
+   int rotated_orig_abs_x, rotated_orig_abs_y;
+   int rotated_new_abs_x, rotated_new_abs_y;
+   int rotated_abs_dx, rotated_abs_dy;
+} MoveVertexInfo;
+
 typedef struct VSelRec {
-   struct ObjRec	* obj;
-   int			n, max_v, * v_index, * x, * y;
-   struct VSelRec	* next, * prev;
+   struct ObjRec		* obj;
+   int				n, max_v, * v_index, * x, * y;
+   struct VSelRec		* next, * prev;
+   /* for moving a vertex -- normally pmvi is NULL */
+   MoveVertexInfo		* pmvi;
 } * VSelRecPtr;
 
 typedef struct MoveSubCmdRec {
@@ -407,6 +450,7 @@ typedef struct CmdRec {
     *       "creator_pid", ":", and "creator_ip" concatenated together.
     */
    int			serialized;
+   char			* serialization_buf;
    int			first_redo_after_deserialize;
    int			skipped;
    struct SubCmdRec	* subcmd;
@@ -435,6 +479,7 @@ typedef struct StkRec {
    float		print_mag;
    int			v_space, grid_shown, move_mode;
    int			rcb_radius, underline_on, underline_y_offset;
+   int			overline_on, overline_y_offset;
    char			name[MAXPATHLENGTH+1], domain[MAXPATHLENGTH+1];
    char			dir[MAXPATHLENGTH+1], sym_dir[MAXPATHLENGTH+1];
    char			* saved_comments;
@@ -489,7 +534,7 @@ typedef struct XPmRec {
    int			fill, real_type, flip, image_w, image_h;
    int			linked_jpeg; /* TRUE if JPEG is linked */
    char			* filename; /* only valid if JPEG is linked */
-   char			tmp_ppm6_fname[MAXPATHLENGTH+1]; /* only valid if JPEG is linked */
+   char			tmp_ppm6_fname[MAXPATHLENGTH+1]; /* used temporarily */
    int			ncolors;
    int			chars_per_pixel;
    int			first_pixel_is_bg;
@@ -504,6 +549,19 @@ typedef struct XPmRec {
    int			cached_w, cached_h, cached_color;
    Pixmap		clip_mask;
    struct XfrmMtrxRec	cached_ctm;
+   /*
+    * If real_type is PPM_TRUE, color_char, color_str, pixels, and data are
+    *         all NULL and ncolors is 0.
+    * The following are only valid if real_type is PPM_TRUE.
+    */
+   int			ppm_data_compress; /* 0x1: jpeg, 0x2: ppm.z */
+   char			* ppm_data; /* can be raw (jpeg) or deflated (ppm.z) */
+   unsigned int		ppm_data_size;
+   char			* ppm_mask_data; /* not used, for future transparency */
+   unsigned int		ppm_mask_size; /* not used, for future transparency */
+   int			has_transparent_color;
+   unsigned char	transparent_color[3]; /* r, g, b */
+   void			* userdata;
 } * XPmRecPtr;
 
 typedef struct WinInfoRec {
@@ -579,6 +637,8 @@ typedef struct PushedFontRec {
    int color;
    int underline_on;
    int underline_y_offset;
+   int overline_on;
+   int overline_y_offset;
    int double_byte, double_byte_mod_bytes, double_byte_vertical;
    struct PushedFontRec *next;
 } * PushedFontRecPtr;
@@ -609,6 +669,12 @@ typedef struct tagTextHighlightInfo {
    int mode; /* one of PAINT_* above */
 } TextHighlightInfo;
 
+typedef struct tagCopyUTF8Info {
+   int single_byte_valid, double_byte_valid;
+   StrSegInfo single_byte_seg;
+   StrSegInfo double_byte_seg;
+} CopyUTF8Info;
+
 typedef struct tagDrawTextInfo {
    /* the following is from miniline.e */
 
@@ -658,6 +724,20 @@ typedef struct tagDrawTextInfo {
    struct PushedFontRec font_info;
 } DrawTextInfo;
 
+typedef struct tagColorBytes {
+   int valid;
+   unsigned char r, g, b;
+} ColorBytes;
+
+typedef struct tagTrueColorInfo {
+   unsigned long r_mask, g_mask, b_mask;
+   unsigned int r_shift, g_shift, b_shift;
+   double dr_maxval, dg_maxval, db_maxval;
+   /* dr_maxval_div255 = dr_maxval/255, etc. */
+   double dr_maxval_div255, dg_maxval_div255, db_maxval_div255;
+   unsigned int num_r_bits, num_g_bits, num_b_bits;
+} TrueColorInfo;
+
 /* ------------------ imageproc stuff ------------------ */
 
 typedef void (ImageMapColorFunc)ARGS_DECL((int nColorIndex, XColor*));
@@ -734,8 +814,8 @@ typedef struct PropertiesRec {
    int			color, width, aw, ah, width_index;
    char			color_str[40], width_spec[40], aw_spec[40], ah_spec[40];
    int			fill, pen, dash, arrow_style, curved, rcb_radius;
-   int			trans_pat;
-   int			text_just, text_sz_unit, v_space, underline_on;
+   int			trans_pat, text_just, text_sz_unit, v_space;
+   int			underline_on, overline_on;
    int			double_byte, text_font, text_style;
    int			transformed;
    struct XfrmMtrxRec	ctm;
@@ -961,6 +1041,16 @@ typedef struct tagPageInfoRec {
    struct PageRec *first_page, *last_page, *cur_page;
 } PageInfo;
 
+/* ------------------ progress stuff ------------------ */
+
+typedef struct tagProgressInfoRec {
+   int total_size;
+   int target_percent;
+
+   char status_buf[MAX_STATUS_BTNS+1][MAXSTRING+1];
+   int one_line_status;
+} ProgressInfo;
+
 /* ------------------ generated_by stuff ------------------ */
 
 typedef struct tagGeneratedByInfoRec {
